
/**
 * CreatDigSig.java
 *
 *
 * Created: Wed Mar 13 16:09:23 2002
 *
 * @author Razi Lotfi-Tabrizi
 * @version
 */

import java.io.*;
import java.util.*;
import java.lang.*;
import java.security.cert.*;
import java.security.*;

public class CreatDigSig  {
    
    private File keystoreFile;
    private KeyStore keystore;
    private String storepass;
    private String keypass;
    private String alias;
    private FileInputStream quelle; 
    private String keystoreType = "JKS";
    private PrivateKey pKey;
    private Signature signature;
    public byte[] signResult;
    private String signatureAlgorithm;

    public CreatDigSig(String[] args) {

	try {
	    keystoreFile = new File(args[0]);
	    storepass = args[1];
	    keypass = args[2];
	    alias = args[3];
	    quelle = new FileInputStream(args[4]);
	}
	catch (Exception e) {
	    System.err.println(e);
	    return;
	}

	try {
	    keystore = KeyStore.getInstance(keystoreType);
	    keystore.load(new BufferedInputStream(new FileInputStream(keystoreFile)),
			  storepass.toCharArray());
	    pKey = (PrivateKey) keystore.getKey(alias, keypass.toCharArray()); 
	}
	catch (Exception e) {
	    System.err.println("Problems with keystore: " + e);
	    return;
	}
	
	if ( pKey.getAlgorithm() == "RSA") {
	    signatureAlgorithm = "SHA1withRSA";
	}
	else {
	    signatureAlgorithm = "SHA1withDSA";
	}
	
	System.err.println("\n" +
			   "Private Key Serial Version UID: " + 
			   pKey.serialVersionUID + "\n" +
			   "Private Key Format:             " + 
			   pKey.getFormat() + "\n" + 
			   "Private Key Algorithm:          " + 
			   pKey.getAlgorithm() + "\n" +
			   "Signature Algorithm:            " + 
			   signatureAlgorithm + "\n");
    }
    
    public void sign() {
	byte[] data = new byte[1024];

	try {
	    signature = Signature.getInstance(signatureAlgorithm);
	    signature.initSign(pKey);
	    
  	    int iii;
	    while( (iii = quelle.read(data)) != -1 ) {
		signature.update(data, 0, iii);
	    }

	    quelle.close();
	    signResult = signature.sign(); 
	}
	catch (Exception e) {
	    System.err.println("ERROR DURING THE SIGNING!\n" + e.toString());
	}  
    }

    public static void main(String[] args) {
	if (args.length != 5) {
	    System.err.println("usage: CreatDigSig <keystore> <storepass> <keypass> <alias> <quelle>");
	    System.err.println("This program can only handle \"JKS\"-keystores!");
	    return;
	}

  	CreatDigSig creatDigSig = new CreatDigSig(args);
	
	creatDigSig.sign();

	sun.misc.BASE64Encoder encoder = new sun.misc.BASE64Encoder();
	String base64 = encoder.encode(creatDigSig.signResult);

	System.err.println("-----BEGIN SIGNATURE-----\n" + 
			   base64 + 
			   "\n-----END SIGNATURE-----\n");

	return;
    }
	

} // CreatDigSig
