/*
 * @(#)SSLClientWithClientAuth.ja
 *
 * Razi Lotfi-Tabrizi
 */


import java.net.*;
import java.io.*;
import javax.net.ssl.*;
import javax.security.cert.X509Certificate;
import java.security.KeyStore;

/*
 * This example shows how to set up a key manager to do client
 * authentication if required by server.
 *
 * This program assumes that the client is not inside a firewall.
 * The application can be modified to connect to a server outside
 * the firewall by following SSLSocketClientWithTunneling.java.
 */
public class SSLClientWithClientAuth {

    public static void main(String[] args) throws Exception {
	String host = null;
	int port = -1;
	String path = null;
	String account = null;
	String accountBASE64 = null;

	for (int i = 0; i < args.length; i++)
	    System.err.println(args[i]);
	
	System.err.println("-----------------------------------------------------------");


	if (args.length < 3) {
	    System.out.println(
		"USAGE: java SSLClientWithClientAuth " +
		"host port requestedfilepath [username:password]");
	    System.exit(-1);
	}

	try {
	    host = args[0];
	    port = Integer.parseInt(args[1]);
	    path = args[2];
	    if (args.length > 3 ) { 
		account = args[3];
		accountBASE64 = new 
		    sun.misc.BASE64Encoder().encode(account.getBytes());
	    }
	} catch (IllegalArgumentException e) {
	     System.out.println("USAGE: java SSLClientWithClientAuth " +
		 "host port requestedfilepath [username:password]");
	     System.exit(-1);
	}

	try {

	    /*
	     * Set up a key manager for client authentication
	     * if asked by the server.  Use the implementation's
	     * default TrustStore and secureRandom routines.
	     */
	    SSLSocketFactory factory = null;
	    try {
		SSLContext ctx;
		KeyManagerFactory kmf;
		TrustManagerFactory tmf;
		KeyStore ks;
		KeyStore ts;
		
		ctx = SSLContext.getInstance(System.getProperty("indigo.httpsProtocols"));

		char[] passphrase = (System.getProperty("indigo.keyStorePassword")).toCharArray();
		kmf = KeyManagerFactory.getInstance(System.getProperty("indigo.keyManagerFactoryAlgorithm"));
		ks = KeyStore.getInstance(System.getProperty("indigo.keyStoreType"));
		ks.load(new FileInputStream(System.getProperty("indigo.keyStore")), passphrase);
		kmf.init(ks, passphrase);

		// TrustStore initialisieren und lesen:
		char[] passphrase1 = (System.getProperty("indigo.trustStorePassword")).toCharArray();
		tmf = TrustManagerFactory.getInstance(System.getProperty("indigo.trustManagerFactoryAlgorithm"));
		ts = KeyStore.getInstance(System.getProperty("indigo.trustStoreType"));
		ts.load(new FileInputStream(System.getProperty("indigo.trustStore")), passphrase1);
		tmf.init(ts);

//  		ctx.init(kmf.getKeyManagers(), null, null);
		ctx.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);

		factory = ctx.getSocketFactory();
	    } catch (Exception e) {
		throw new IOException(e.getMessage());
	    }

	    SSLSocket socket = (SSLSocket)factory.createSocket(host, port);

	    /*
	     * send http request
	     *
	     * See SSLSocketClient.java for more information about why
	     * there is a forced handshake here when using PrintWriters.
	     */
	    socket.startHandshake();

	    PrintWriter out = new PrintWriter(
				  new BufferedWriter(
				  new OutputStreamWriter(
     				  socket.getOutputStream())));
	    out.println("GET " + path + " HTTP/1.1");
	    if (args.length > 3 ) { 
		out.println("Authorization: Basic " + accountBASE64);
	    }
	    out.println();
	    out.flush();

	    /*
	     * Make sure there were no surprises
	     */
	    if (out.checkError())
		System.out.println(
		    "SSLSocketClient: java.io.PrintWriter error");

	    /* read response */
	    BufferedReader in = new BufferedReader(
				    new InputStreamReader(
				    socket.getInputStream()));

	    String inputLine;

	    while ((inputLine = in.readLine()) != null)
		System.out.println(inputLine);

	    in.close();
	    out.close();
	    socket.close();

	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
}
