/*
 *  Base.java  $Revision: 2.7 $
 *
 *  $Date: 1999/12/21 14:53:04 $
 *  $Author: cristian $
 *
 *  Copyright (C) 1999  Christian Moench
 *
 *
 */
import java.io.*;
import java.net.*;
import java.util.*;
import javax.net.ssl.*;
import javax.net.*;
import java.security.KeyStore;
import javax.security.cert.X509Certificate;


public class Base {

    private static String lastError = "";


    /*
     * Invoke the document method `aMethod' on document `anId' on
     * server `aServer', port `aPort' with the Arguments `args'.
     */
    public static String[] invoke(String aServer, int aPort,
				  String anId,
				  String aMethod, String[] args) {	

	// Assemble the path string to reflect an invokation.
	String path;
	path = "/diglib/invoke/" + anId + "?" + aMethod;
	if (args != null) {
	    for (int i = 0; i < args.length; i++) {
		path = path + ("&" + args[i]);
	    }
	}

        if (IndigoConfig.secureExtension()) {
	    HostnameVerifier hv = new HostnameVerifier() {
		    public boolean verify(String hostname, SSLSession session) {
			// System.out.println("WARNING: HTTPS hostname wrong.");
			return true;
		    }
		};
	    HttpsURLConnection.setDefaultHostnameVerifier(hv);
	}
	
	try {
	    URL invokeURL;
	    HttpURLConnection con;

	    // Open the connection
	    if (IndigoConfig.secureExtension()) {
		invokeURL = new URL("https", aServer, aPort, path);
		con = (HttpsURLConnection)invokeURL.openConnection();
		((HttpsURLConnection)con).setSSLSocketFactory(IndigoSSLContext.getStaticSSLSocketFactory());
	    }
	    else {
		invokeURL = new URL("http", aServer, aPort, path);
		con = (HttpURLConnection)invokeURL.openConnection();
	    }
	    
	    // Check for HTTP error
	    if (con.getResponseCode() != HttpURLConnection.HTTP_OK) {
		lastError = con.getResponseMessage();
		System.err.println("Error: " + lastError);
		return null;
	    }
	    lastError = "Ok";

	    // Read the content
	    BufferedReader in;
	    String         line;
	    Vector         lines;

	    in = new BufferedReader(
		new InputStreamReader(con.getInputStream()));
	    lines = new Vector();
	    line = in.readLine();

	    while (line != null) {
		lines.addElement(line);
		line = in.readLine();
	    }

	    // Build result array and return it
	    String[] result;
	    result = new String[lines.size()];
	    lines.copyInto(result);
	    return result;
	}
	catch (Exception e) {
	    lastError = e.toString();
	    System.err.println("Error: " + e.toString());
	    e.printStackTrace(System.err);

	    String[] result = new String[2];
	    result[0] = "Success: false";
	    result[1] = "Error: " + e.toString();
	    return result;
	}
    }


    public static InputStream invokeAsync(String aServer, int aPort,
					  String anId,
					  String aMethod, String[] args) {

	// Assemble the path string to reflect an invokation.
	String path;
	path = "/diglib/invoke/" + anId + "?" + aMethod;
	if (args != null) {
	    for (int i = 0; i < args.length; i++) {
		path = path + ("&" + args[i]);
	    }
	}

        if (IndigoConfig.secureExtension()) {
	    HostnameVerifier hv = new HostnameVerifier() {
		    public boolean verify(String hostname, SSLSession session) {
			// System.out.println("WARNING: HTTPS hostname wrong.");
			return true;
		    }
		};
	    HttpsURLConnection.setDefaultHostnameVerifier(hv);
	}
	
	try {
	    URL invokeURL;
	    HttpURLConnection con;

	    // Open the connection
	    if (IndigoConfig.secureExtension()) {
		invokeURL = new URL("https", aServer, aPort, path);
		con = (HttpsURLConnection)invokeURL.openConnection();
		((HttpsURLConnection)con).setSSLSocketFactory(IndigoSSLContext.getStaticSSLSocketFactory());
	    }
	    else {
		invokeURL = new URL("http", aServer, aPort, path);
		con = (HttpURLConnection)invokeURL.openConnection();
	    }

	    // Check for HTTP error
	    if (con.getResponseCode() != HttpURLConnection.HTTP_OK) {
		lastError = con.getResponseMessage();
		System.err.println("Error: " + lastError);
		return null;
	    }
	    lastError = "Ok";
	    return con.getInputStream();
	}
	catch (Exception e) {
	    lastError = e.toString();
	    System.err.println("Error: " + e.toString());
	    e.printStackTrace(System.err);

	    String[] result = new String[2];
	    result[0] = "Success: false";
	    result[1] = "Error: " + e.toString();
	    return null;
	}
    }


    public static String s_transport(String aServer, int aPort,
				     String anId) {
	// Get the document directory
	String documentDir;
	documentDir = System.getProperty("indigo.documentdir");
	if (documentDir == null) {
	    lastError = "property indigo.documentdir not found";
	    return null;
	}

        if (IndigoConfig.secureExtension()) {
	    HostnameVerifier hv = new HostnameVerifier() {
		    public boolean verify(String hostname, SSLSession session) {
			// System.out.println("WARNING: HTTPS hostname wrong.");
			return true;
		    }
		};
	    HttpsURLConnection.setDefaultHostnameVerifier(hv);
	}
	
	try {
	    // Build URL and open the connection
	    URL                anUrl;
	    HttpURLConnection aConnection;
	    BufferedReader     in;
	    String             line;

	    // Open the connection
	    if (IndigoConfig.secureExtension()) {
		anUrl = new URL("https", aServer, aPort, "/diglib/deliver");
		aConnection = (HttpsURLConnection)anUrl.openConnection();
		((HttpsURLConnection)aConnection).
		    setSSLSocketFactory(IndigoSSLContext.getStaticSSLSocketFactory());
	    }
	    else {
		anUrl = new URL("http", aServer, aPort, "/diglib/deliver");
		aConnection = (HttpURLConnection)anUrl.openConnection();
	    }
	    aConnection.setRequestMethod("POST");
	    aConnection.setDoOutput(true);
	    aConnection.connect();

	    PrintWriter out;
	    out = new PrintWriter(aConnection.getOutputStream());

	    String boundary = "sdoxcccjkh5lkd93450982ljis5s0d789234";
	    out.println("Content-Type: application/x-metadoc;");
	    out.println(" boundary=" + boundary);
	    out.println("");
	    out.println("--" + boundary);
	    out.println("Content-Type: application/x-metadoc-attributes");
	    out.println("");
	    in = new BufferedReader(
		new FileReader(documentDir + "/" + anId + "/attributes"));
	    line = in.readLine();
	    while (line != null) {
		out.println(line);
		line = in.readLine();
	    }
	    out.println("--" + boundary);
	    out.println("Content-Type: application/x-metadoc-methods");
	    out.println("");
	    in = new BufferedReader(
		new FileReader(documentDir + "/" + anId + "/methods"));
	    line = in.readLine();
	    while (line != null) {
		out.println(line);
		line = in.readLine();
	    }
	    out.println("--" + boundary);
	    out.println("Content-Type: application/x-metadoc-content");
	    out.println("");
	    in = new BufferedReader(
		new FileReader(documentDir + "/" + anId + "/content"));
	    line = in.readLine();
	    while (line != null) {
		out.println(line);
		line = in.readLine();
	    }
	    out.println("--" + boundary + "--");
	    out.flush();

	    // Check for HTTP error
	    if (aConnection.getResponseCode() != HttpURLConnection.HTTP_OK) {
		lastError = aConnection.getResponseMessage();
		System.err.println("d_transport failed: " + lastError);
		return null;
	    }
	    lastError = "Ok";
	    
	    // Read result and extract document id
	    in = new BufferedReader
		(new InputStreamReader
		    (aConnection.getInputStream()));
	    line = in.readLine();
	    while (line != null) {
		if (line.startsWith("document-id: ")) {
		    return line.substring(13).trim();
		}
		line = in.readLine();
	    }
	    aConnection.disconnect();
	    return null;
	}
	catch (Exception e) {
	    lastError = e.toString();
	    System.err.println("s_transport: " + e.toString());
	    e.printStackTrace(System.err);
	    return null;
	}
    }


    public static String d_transport(String aServer, int aPort,
				     String anId) {

	// Get the document directory
	String documentDir;
	documentDir = System.getProperty("indigo.documentdir");
	if (documentDir == null) {
	    lastError = "property indigo.documentdir not found";
	    return null;
	}

        if (IndigoConfig.secureExtension()) {
	    HostnameVerifier hv = new HostnameVerifier() {
		    public boolean verify(String hostname, SSLSession session) {
			// System.out.println("WARNING: HTTPS hostname wrong.");
			return true;
		    }
		};
	    HttpsURLConnection.setDefaultHostnameVerifier(hv);
	}
	
	try {
	    // Build URL and open the connection
	    URL                anUrl;
	    HttpURLConnection  aConnection;
	    BufferedReader     in;
	    String             line;

	    // Open the connection
	    if (IndigoConfig.secureExtension()) {
		anUrl = new URL("https", aServer, aPort, "/diglib/deliver");
		aConnection = (HttpsURLConnection)anUrl.openConnection();
		((HttpsURLConnection)aConnection).
		    setSSLSocketFactory(IndigoSSLContext.getStaticSSLSocketFactory());
	    }
	    else {
		anUrl = new URL("http", aServer, aPort, "/diglib/deliver");
		aConnection = (HttpURLConnection)anUrl.openConnection();
	    }
	    aConnection.setRequestMethod("POST");
	    aConnection.setDoOutput(true);
	    aConnection.connect();

	    PrintWriter out;
	    out = new PrintWriter(aConnection.getOutputStream());

	    String boundary = "sdoxcccjkh5lkd93450982ljis5s0d789234";
	    out.println("Content-Type: application/x-metadoc;");
	    out.println(" boundary=" + boundary);
	    out.println("");
	    out.println("--" + boundary);
	    out.println("Content-Type: application/x-metadoc-attributes");
	    out.println("");
	    in = new BufferedReader(
		new FileReader(documentDir + "/" + anId + "/attributes"));
	    line = in.readLine();
	    while (line != null) {
		out.println(line);
		line = in.readLine();
	    }
	    out.println("--" + boundary);
	    out.println("Content-Type: application/x-metadoc-methods");
	    out.println("");
	    in = new BufferedReader(
		new FileReader(documentDir + "/" + anId + "/methods"));
	    line = in.readLine();
	    while (line != null) {
		out.println(line);
		line = in.readLine();
	    }
	    out.println("--" + boundary);

	    // Transfer empty metadocument
	    out.println("Content-Type: application/x-metadoc-content");
	    out.println("");
	    out.println("");
	    out.println("--" + boundary + "--");
	    out.flush();

	    // Check for HTTP error
	    if (aConnection.getResponseCode() != HttpURLConnection.HTTP_OK) {
		lastError = aConnection.getResponseMessage();
		System.err.println("d_transport failed: " + lastError);
		return null;
	    }
	    lastError = "Ok";
	    
	    // Read result and extract document id
	    in = new BufferedReader
		(new InputStreamReader
		    (aConnection.getInputStream()));
	    line = in.readLine();
	    while (line != null) {
		if (line.startsWith("document-id: ")) {
		    return line.substring(13).trim();
		}
		line = in.readLine();
	    }
	    aConnection.disconnect();
	    return null;
	}
	catch (Exception e) {
	    lastError = e.toString();
	    System.err.println("d_transport: " + e.toString());
	    e.printStackTrace(System.err);
	    return null;
	}
    }


    public static String[] s_transport(String aServer, int aPort,
				       String anId,
				       String aMethod, String[] args) {

	String newId = s_transport(aServer, aPort, anId); 
	String[] result = invoke(aServer, aPort, newId, aMethod, args);
	String[] realresult = new String[result.length + 1];
	System.arraycopy(result, 0, realresult, 1, result.length);
	realresult[0] = "document-id: " + newId;
	return 	realresult;
    }


    public static String[] d_transport(String aServer, int aPort,
				       String anId,
				       String aMethod, String[] args) {

	String newId = d_transport(aServer, aPort, anId); 
	String[] result = invoke(aServer, aPort, newId, aMethod, args);
	String[] realresult = new String[result.length + 1];
	System.arraycopy(result, 0, realresult, 1, result.length);
	realresult[0] = "document-id: " + newId;
	return 	realresult;
    }


    public static String getLastError() {
	return lastError;
    }
}
