
/**
 * NetSSL.java
 *
 *
 * Created: Thu Feb  14 18:27:34 2002
 *
 * @author Razi Lotfi-Tabrizi
 * @version
 */

import java.net.*;
import java.lang.*;
import java.io.*;
import java.security.KeyStore;
import javax.net.*;
import javax.net.ssl.*;
import javax.security.cert.X509Certificate;

public class NetSSL {
    private String thisDocumentId;
    private Socket currentClientSocket;
    private String indigoPort;
    private String hostname;
    private String hostaddress;
    private MethodThread  mt;

    private SSLServerSocket     netSSLServerSocket;
    private SSLSocket           netSSLSocket;

    private IndigoSSLServerSocket indigoSSLServerSocket;
    private IndigoSSLSocket       indigoSSLSocket;

    private int                 netPort;     // Die Port-Adresse des Servers
    private String              netHostname; // Der Hostname des Servers
    private InetAddress         netAddress;  // Die Adresse des Servers
    
    private boolean       	IAmServer;

    
    // Ist fuer das ServerSSLSocket erzeugendes Programm (also z.B. Server) gedacht 
    public NetSSL(int portArg) {
  	mt = (MethodThread)Thread.currentThread();
	thisDocumentId = (String)mt.documentId;
	this.indigoPort = IndigoConfig.portToString();
	this.hostname = IndigoConfig.hostnameToString();
	this.hostaddress = IndigoConfig.hostaddressToString();
	
	try {
	    indigoSSLServerSocket = new IndigoSSLServerSocket();
	    netSSLServerSocket = (SSLServerSocket)indigoSSLServerSocket.createServerSocket(portArg);

	    netPort = netSSLServerSocket.getLocalPort();
	    netAddress = netSSLServerSocket.getInetAddress();
	    netHostname = netAddress.getHostAddress();
	    IAmServer = true;
	}
	catch (Exception e)  {
	    System.err.println(e.toString());
	}
    }

    // Auch fuer den Server
    public NetSSL() {
	this(0);
    }

    // Dieser Konstruktor ist fuer jenes Programm gedacht, das auf 
    // bestehendes Socket zugreift (also z.B. Client). 
    public NetSSL(String arg_host, int arg_port) {
  	mt = (MethodThread)Thread.currentThread();
	thisDocumentId = (String)mt.documentId;
	this.indigoPort = IndigoConfig.portToString();
	this.hostname = IndigoConfig.hostnameToString();
	this.hostaddress = IndigoConfig.hostaddressToString();

	try {
	    indigoSSLSocket  = new IndigoSSLSocket();
	    netSSLSocket   = (SSLSocket)indigoSSLSocket.createSocket(arg_host, arg_port);
	}
	catch (Exception e)  {
	    System.err.println(e.toString());
	}
	netPort = arg_port;
	netHostname = arg_host;
	try {
	    netAddress = InetAddress.getByName(arg_host);
	}
	catch (Exception e) {
	    System.err.println(e.toString());
	}
	IAmServer = false;
    }    

    public SSLServerSocket getSSLServerSocket() {
	if (IAmServer) {
	return(this.netSSLServerSocket);
	} 
	else {
	    return(null);
	}
    }

    public SSLSocket getSSLSocket() {
	if (!IAmServer) {
	    return(this.netSSLSocket);
	} 
	else {
	    return(null);
	}
    }

    public int getNetSSLPort() {
	return(netPort);
    }
    
    public void setNetSSLPort(int arg_port) {
	netPort = arg_port;
    }
    
    public String getNetSSLHostname() {
	return(netHostname);
    }
 
    public InetAddress getNetSSLAddress() {
	return(netAddress);
    }
 
} // NetSSL

