/*
 *  Attributes.java  $Revision: 2.6 $
 *
 *  $Date: 1999/12/19 16:14:51 $ $Author: razi $
 *
 *  Copyright (C) 1999  Christian Moench
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Christian Moench
 *  Dept. of Computer Science
 *  Johann Wolfgang Goethe-University
 *  60325 Frankfurt, Germany
 *  Tel.  +49 69 79828358
 *  Fax.  +49 69 79823340
 *
 *  $Log: Attributes.java,v $
 *  Revision 2.6  1999/12/19 16:14:51  razi
 *  Including IndigoConfig.java
 *
 *  Revision 2.2  1999/12/15 16:37:20  cristian
 *  cleanups
 *
 *  Revision 1.1.1.1  1999/07/28 15:55:58  cristian
 *  Initial import
 *
 *
 */
import java.io.*;
import java.lang.*;
import java.util.*;



public class Attributes {


    Hashtable theAttributes;
    

    /*
     * Read the attribute specification from a buffered reader.
     */
    public void readFrom(BufferedReader reader) throws IOException {
	int             index;
	String          line, name, value;
	StringTokenizer splitter;
	String          multiLineBegin =  "<MULTILINE>";
	String          multiLineEnd   = "</MULTILINE>";


	line = reader.readLine();
	while (line != null) {
	    splitter = new StringTokenizer(line.trim());
	    if (splitter.countTokens() < 2) {
  		throw new IOException("Bad attribute line: " + line);
	    }

	    try {
		// Read attribute name
		name = splitter.nextToken();
		
		// Read attribute value
		value = "";
		while (splitter.hasMoreTokens()) {
		    value = value + splitter.nextToken() + " ";
		}
	    }
	    catch (NoSuchElementException e) {
		e.printStackTrace(System.err);
		return;
	    }
	    
	    // handle MultiLine value such certificates
  	    if ((value.trim()).equalsIgnoreCase(multiLineBegin.trim())) {
		line = reader.readLine();
		value = multiLineBegin + "\n";
		while ((line != null) && 
		       (!((line.trim()).equalsIgnoreCase(multiLineEnd.trim())))) {
		    value = value + line + "\n";
		    line = reader.readLine();
		}
  		if (line == null) {
  		    throw new IOException("Bad attribute muliline: " + line);
  		} else if ((line.trim()).equalsIgnoreCase(multiLineEnd.trim())) {
		    value = value + line;
		}
	    }

	    theAttributes.put(name.trim(), value.trim());
	    line = reader.readLine();
	}
    }


    /*
     * Write the attribute specification to a buffered reader.
     */
    public void writeTo(PrintWriter writer) throws IOException {
	String      name;
	Enumeration attributeNames;
	
	attributeNames = theAttributes.keys();
	while (attributeNames.hasMoreElements()) {
	    try {
		name = (String)attributeNames.nextElement();
	    }
	    catch (NoSuchElementException e) {
		e.printStackTrace(System.err);
		return;
	    }
	    writer.println(name + " " + (String)theAttributes.get(name));
	}
    }


    public Attributes() {
	theAttributes = new Hashtable();
	return;
    }
}
