import java.io.*;
import java.util.*;


class BoundaryReader extends BufferedReader {

    protected Stack   theBoundaries;
    protected String  currentBoundary;
    protected boolean onBoundary;


    public BoundaryReader(Reader r) {
	super(r);
	onBoundary      = false;
	currentBoundary = "";
	theBoundaries   = new Stack();
    }


    public boolean pushBoundary(String boundary) {

	if (onBoundary == true) {
	    return false;
	}
	theBoundaries.push(boundary);
	return true;
    }


    public boolean isBoundary() {
	String boundary = (String)theBoundaries.peek();
	return currentBoundary.equals("--" + boundary);
    }


    public boolean isFinalBoundary() {
	String boundary = (String)theBoundaries.peek();
	return currentBoundary.equals("--" + boundary + "--");
    }


    public void skipFinalBoundary(String bound) throws IOException {
	String boundary = (String)theBoundaries.peek();

	if (onBoundary == true
	    && currentBoundary.equals("--" + boundary + "--")) {
	    currentBoundary = "";
	    onBoundary = false;
	}
	return;
    }


    public void skipBoundary() throws IOException {
	String boundary = (String)theBoundaries.peek();

	if (onBoundary == true
	    && currentBoundary.equals("--" + boundary)) {
	    currentBoundary = "";
	    onBoundary = false;
	}
	return;
    }


    public String readLine() throws IOException {

	if (onBoundary == true) {
	    return null;
	}

	if (theBoundaries.empty() == true) {
	    return super.readLine();
	}

	String boundary = (String)theBoundaries.peek();
	String line = super.readLine();
	if (line.startsWith("--" + boundary)) {
	    onBoundary = true;
	    currentBoundary = line;
	    return null;
	}
	return line;
    }
}
