/*
 *  DispatchedStream.java  $Revision: 2.1 $
 *
 *  $Id: DispatchedStream.java,v 2.1 1999/12/21 14:53:34 cristian Exp $
 *  $Date: 1999/12/21 14:53:34 $
 *
 *  Copyright (C) 1999  Christian Moench
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Christian Moench
 *  Dept. of Computer Science
 *  Johann Wolfgang Goethe-University
 *  60325 Frankfurt, Germany
 *  Tel.  +49 69 79828358
 *  Fax.  +49 69 79823340
 *
 *  $Log: DispatchedStream.java,v $
 *  Revision 2.1  1999/12/21 14:53:34  cristian
 *  added multiplexed standard output for document methods
 *
 *
 */


import java.io.*;
import java.util.*;
import java.lang.*;


public class DispatchedStream extends PrintStream {

    private Hashtable   groupOutput;
    private PrintStream outputStream;

    public DispatchedStream(PrintStream out) {
	super(out);
	groupOutput = new Hashtable();
	outputStream = out;
    }

    public void write(int b) {
	ThreadGroup aGroup = Thread.currentThread().getThreadGroup();
	if (groupOutput.containsKey(aGroup)) {
	    ((PrintWriter)groupOutput.get(aGroup)).write(b);
	} else {
	    outputStream.write(b);
	}
    }

    public void write(byte[] buf, int off, int len) {
	ThreadGroup aGroup = Thread.currentThread().getThreadGroup();
	if (groupOutput.containsKey(aGroup)) {
	    ((PrintStream)groupOutput.get(aGroup)).write(buf, off, len);
	} else {
	    outputStream.write(buf, off, len);
	}
    }

    public void addGroupOutput(ThreadGroup aGroup, PrintStream aStream) {
	groupOutput.put(aGroup, aStream);
    }

    public void removeGroupOutput(ThreadGroup aGroup) {
	groupOutput.remove(aGroup);
    }
}
