import java.io.*;
import java.lang.*;
import java.util.*;


class Header {

    protected BufferedReader source;
    protected String         currentLine, nextLine;
    protected int            currentChar;
    protected Hashtable      theHeaders;
    protected Vector         theComments;


    // Exceptions generated by thins class
    public class ParseException extends Exception {
	public ParseException(String message) {
	    super(message);
	}
    }


    protected void parseLongLine(String line)
	    throws IOException, ParseException {
	StringReader reader;

	// Read the name
	reader = new StringReader(line);
	currentChar = reader.read();

	if (Character.isLetter((char)currentChar) == false) {
	    throw new ParseException("Bad name in line: " + line + "\n");
	}

	String name = "";
	while (Character.isLetterOrDigit((char)currentChar) == true
	       || currentChar == '-') {
	    name = name + (char)currentChar;
	    currentChar = reader.read();
	    if (currentChar == -1) {
		throw new ParseException("Early end in line: " + line + "\n");
	    }
	}


	if (currentChar != ':') {
	    throw new ParseException("Expected `:' instead of `"
				     + (char)currentChar
				     + "' in line: " + line + "\n");
	}
	currentChar = reader.read();
	if (currentChar == -1) {
	    throw new ParseException("Early end in line: " + line + "\n");
	}

	while (currentChar == ' ' || currentChar == '\t') {
	    currentChar = reader.read();
	}

	// Read value until end of line or comment starter
	String value = "";
	while (currentChar != ';' && currentChar != -1) {
	    value = value + (char)currentChar;
	    currentChar = reader.read();
	}
	theHeaders.put(name.toLowerCase(), value);
	IndigoOut.errPrintln("   >> " + name.toLowerCase() + " " + value);

	// Read comments as long as we see a comment starter.
	while (currentChar == ';') {
	    currentChar = reader.read();
	    String comment = "";
	    while (currentChar != ';' && currentChar != -1) {
		comment = comment + (char)currentChar;
		currentChar = reader.read();
	    }
	    theComments.addElement(comment.trim());
	}
    }


    protected void parseLine() throws IOException, ParseException {
	String line;

	if (currentLine.startsWith(" ") || currentLine.startsWith("\t")) {
	    throw new ParseException("Bad header line: " + currentLine + "\n");
	}

	// Connect multiline-lines
	line = currentLine;
	while (nextLine != null && (nextLine.trim().length() > 0)
	       && (nextLine.startsWith(" ") || nextLine.startsWith("\t"))) {
	    line = line + nextLine;
	    currentLine = nextLine;
	    nextLine = source.readLine();
	}
	parseLongLine(line);
	return;
    }


    public Header(BufferedReader s) throws IOException, ParseException {
	source = s;

	theHeaders  = new Hashtable();
	theComments = new Vector();
	nextLine = source.readLine();
	while (nextLine != null && (nextLine.trim().length() > 0)) {
	    currentLine = nextLine;
	    nextLine = source.readLine();
	    parseLine();
	}
	return;
    }


    public String getValue(String name) {
	return (String)theHeaders.get(name);
    }


    public boolean containsKey(String key) {
	return (boolean)theHeaders.containsKey(key);
    }


    public boolean containsValue(String value) {
 	return (boolean)theHeaders.containsValue(value);
    }

    public String getAuthorizationValueBASE64() {
	String authorizationValueBASE64;
	String tmpValue;

	if (this.containsKey("authorization")) {
	    tmpValue = (this.getValue("authorization")).trim();

	    if (tmpValue.startsWith("Basic")) {
		authorizationValueBASE64 = (tmpValue.substring(5)).trim();
		return authorizationValueBASE64;
	    }
	    else {
		return null;
	    }
	} 
	else {
	    return null;
	}
    }

    public Vector getComments() {
	return theComments;
    }


    public String getBoundary() {
	String      comment;
	Enumeration commentList;

	commentList = theComments.elements();
	while (commentList.hasMoreElements()) {
	    try {
		comment = (String)commentList.nextElement();
	    }
	    catch (NoSuchElementException e) {
		IndigoOut.errPrintln("Header#getBoundary: " + e.toString());
		return null;
	    }
	    if (comment.startsWith("boundary=")) {
		String boundary = comment.substring("boundary=".length());
		boundary.trim();
		if (boundary.startsWith("\"") || boundary.startsWith("'")) {
		    boundary = boundary.substring(1);
		}
		if (boundary.endsWith("\"") || boundary.endsWith("'")) {
		    boundary = boundary.substring(0, boundary.length() - 1);
		}
		if (boundary.length() == 0) {
		    IndigoOut.errPrintln("Header#getBoundary: Empty boundary: " + boundary);
		    return null;
		}
		return boundary;
	    }
	}
	return null;
    }
}
