/**
 * IndigoConfig.java
 *
 *
 * Created: Tue Nov 30 16:28:59 1999
 *
 * @author Razi Lotfi-Tabrizi
 * @version
 */

import java.net.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import java.security.KeyStore;
import javax.net.*;
import javax.net.ssl.*;
import javax.security.cert.X509Certificate;


public class IndigoConfig  {
    private  static String cf;  // der vollstaendige Name der Konfigdatei.
    private  static String documentRoot;
    private  static String indigoDir;
    private  static String port;
    private  static int    portInt;	
    private  static String timeout;
    private  static int    timeoutInt;
    private  static String hostname;
    private  static String hostaddress;
    private  static String packages;
    private  static String keyStore;
    private  static String keyStorePassword;
    private  static String keyStorePrivateKeyPassword;
    private  static String keyStoreType;
    private  static String keyManagerFactoryAlgorithm;
    private  static String trustStore;
    private  static String trustStorePassword;
    private  static String trustStoreType;
    private  static String trustManagerFactoryAlgorithm;
    private  static String httpsProtocols;

    private  static boolean needClientAuth;
    private  static boolean wantClientAuth;
    private  static String  clientAuth;

    private  static boolean needMethodVerify;
    private  static String  needMethodVerifyStr;

    private  static boolean verboseMode;
    private  static String  verboseModeStr;

    private  static String    trueAccesscontrolllist;
    private  static String    maybeAccesscontrolllist;
    private  static String    falseAccesscontrolllist;
    private  static String    currentAccesscontrolllist;

    private  static String    superUserName;
    private  static String    superUserPassword;

    private  static boolean secureExtension;
    private  static String  secureExtensionStr; 

    static 
    {
	String stringTmp;
	
	//////////////////////////////////////////////////////////////////////////
	// Diese Werte werden uebernommen, falls, weder KonfigDatei existiert,  // 
	// noch irgendwelche Variable beim Start des Programms gesetzt wurden.  //
	documentRoot="/usr/tmp/indigo";  //09876? Was passiert hier bei Windows 
	indigoDir=(System.getProperty("user.home")  + 
		 File.separator + "jdl" + File.separator + "server");
	port="7333";
        portInt=Integer.parseInt(port);
	timeout="1000";
	timeoutInt=Integer.parseInt(timeout);
	packages="io base";
	cf = indigoDir + File.separator + "indigo.config" ;

	keyStore = indigoDir + File.separator + "indigo.keystore";
	keyStorePassword = "InfBib";
	keyStorePrivateKeyPassword = "InfBib";
	keyStoreType = "JKS";
	keyManagerFactoryAlgorithm = "SunX509";
	trustStore = indigoDir + File.separator + "indigo.truststore";
	trustStorePassword = "InfBib";
	trustStoreType = "JKS";
	trustManagerFactoryAlgorithm = "SunX509";
	httpsProtocols = "SSL";
	needClientAuth = true;
	wantClientAuth = false;
	clientAuth = "maybe";
	needMethodVerify = false;
	needMethodVerifyStr = "false";
	verboseMode = false;
	verboseModeStr = "false";
	secureExtensionStr = "false";
	secureExtension = false;
	    
	trueAccesscontrolllist    = "(all)(invoke,attributes,documents,packages)(none)";
	maybeAccesscontrolllist   = "(all)(invoke,attributes,documents,packages)(attributes,documents,packages)";
	falseAccesscontrolllist   = "(all)(invoke,attributes,documents,packages)(invoke,attributes,documents,packages)";
	currentAccesscontrolllist = maybeAccesscontrolllist;
	////////////////////////////////////////////////////////////////////////////////////


	
	// IP-Adresse und Hostname ermitteln. //
	try {
	    InetAddress addr = InetAddress.getLocalHost();
	    hostname = addr.getHostName();
	    hostaddress = addr.getHostAddress();
	} 
	catch (UnknownHostException e) {
	    System.err.println(e.toString());
	    System.exit(1);
	}
	////////////////////////////////////////


	////////////////////////////////////////////////////////////////////
	// Der vollstaendige Name der KonfigDatei ermitteln               //
	// Es wird nach dieser Reihenfolge nach der KonfigDatei gesucht:  //
	// 1) Variable, die man beim Starten des Programms setzt.         //
	// 2) Im Homeverzeichnis also "~/.indigo.config".                   //
	// 3) Im Verzeichnis des Programms also $indigoDir/indigo.config      //
	stringTmp = null;
	stringTmp = System.getProperty("indigo.configfile");
	File cff0 = new File("");
	if (stringTmp != null) {
	    cff0= new File(stringTmp);
	}	
	
	File cff1= new File(System.getProperty("user.home")  + File.separator + ".indigo.config");
	File cff2= new File(cf);
	if ((stringTmp != null) && (cff0.exists())) {
	    cf = stringTmp;
	}
	else if (cff1.exists())	{
	    cf = System.getProperty("user.home")  + File.separator + ".indigo.config";
	}
	else if (cff2.exists()) {
	}
	else {
	    cf="";
	    System.err.println("\nThe server can not find any config file!\n");
	} 
	////////////////////////////////////////////////////////////////////
	
	
	
	// Die KonfigDatei parsen!
	try {
	    // Hier werden auch die Kommentare und Leerzeilen eliminiert!
	    Properties prop = new Properties();
	    FileInputStream fis = new FileInputStream(cf);
	    prop.load(fis);
		
	    port                = prop.getProperty("indigo.port");
	    portInt             = Integer.parseInt(port);
	    timeout             = prop.getProperty("indigo.timeout");
	    timeoutInt          = Integer.parseInt(timeout);
	    packages            = prop.getProperty("indigo.packages");	    
	    documentRoot        = prop.getProperty("indigo.documentdir");	    
	    indigoDir             = prop.getProperty("indigo.serverdir");   

	    keyStore            = prop.getProperty("indigo.keyStore");
	    keyStorePassword    = prop.getProperty("indigo.keyStorePassword");
	    keyStorePrivateKeyPassword = 
		prop.getProperty("indigo.keyStorePrivateKeyPassword");
	    keyStoreType        = prop.getProperty("indigo.keyStoreType");
	    keyManagerFactoryAlgorithm = 
		prop.getProperty("indigo.keyManagerFactoryAlgorithm");
	    trustStore          = prop.getProperty("indigo.trustStore");
	    trustStorePassword  = prop.getProperty("indigo.trustStorePassword");
	    trustStoreType      = prop.getProperty("indigo.trustStoreType");
	    trustManagerFactoryAlgorithm = 
		prop.getProperty("indigo.trustManagerFactoryAlgorithm");
	    httpsProtocols      = prop.getProperty("indigo.httpsProtocols");
  	    clientAuth          = prop.getProperty("indigo.needClientAuth");
	    needMethodVerifyStr = prop.getProperty("indigo.needMethodVerify");
	    verboseModeStr      = prop.getProperty("indigo.verboseMode"); 
	    trueAccesscontrolllist      = prop.getProperty("indigo.trueAccesscontrolllist");
	    maybeAccesscontrolllist     = prop.getProperty("indigo.maybeAccesscontrolllist");
	    falseAccesscontrolllist     = prop.getProperty("indigo.falseAccesscontrolllist");
	    
	    superUserName       = prop.getProperty("indigo.superUserName");
	    superUserPassword   = prop.getProperty("indigo.superUserPassword");
	    secureExtensionStr  = prop.getProperty("indigo.secureExtension"); 
	    //= prop.getProperty("");	    
	} 
	catch (IOException e) {
	    System.err.println(e.toString() + "config file\n");
	    //System.exit(1);
	}



	/////////////////////////////////////////////////////////////////////////////	
	// Variable, die man beim Starten des skripts "server/server" setzt        //
	stringTmp = null;
	stringTmp = System.getProperty("indigo.port");
	if (stringTmp != null) {
	    port = stringTmp;
	    portInt=Integer.parseInt(port);
	}

	stringTmp = null;
	stringTmp = System.getProperty("indigo.timeout");
	if (stringTmp != null) {
	    timeout = stringTmp;
	    timeoutInt=Integer.parseInt(timeout);
	}	

	stringTmp = null;
	stringTmp = System.getProperty("indigo.documentdir");
	if (stringTmp != null) {
	    documentRoot = stringTmp;
	}
	
	stringTmp = null;
	stringTmp = System.getProperty("indigo.serverdir");
	if (stringTmp != null) {
	    indigoDir = stringTmp;
	}

	stringTmp = null;
	stringTmp = System.getProperty("indigo.packages");
	if (stringTmp != null) {
	    packages = stringTmp;
	}

	stringTmp = null;
	stringTmp = System.getProperty("indigo.verboseMode");
	if (stringTmp != null) {
	    verboseModeStr = stringTmp;
	}
	if (verboseModeStr.equalsIgnoreCase("true")) {
	    verboseMode = true;
	}
	else if (verboseModeStr.equalsIgnoreCase("false")) {
	    verboseMode = false;
	}
	
	stringTmp = null;
	stringTmp = System.getProperty("indigo.secureExtension");
	if (stringTmp != null) {
	    secureExtensionStr = stringTmp;
	}
	if (secureExtensionStr.equalsIgnoreCase("true")) {
	    secureExtension = true;
	}
	else if (secureExtensionStr.equalsIgnoreCase("false")) {
	    secureExtension  = false;
	}


	// falls ueberhaupt Sicherheit wichtig ist!!!
	if (secureExtension) {

	    stringTmp = null;
	    stringTmp = System.getProperty("indigo.keyStore");
	    if (stringTmp != null) {
		keyStore = stringTmp;
	    }

	    stringTmp = null;
	    stringTmp = System.getProperty("indigo.keyStorePassword");
	    if (stringTmp != null) {
		keyStorePassword = stringTmp;
	    }
	    
	    stringTmp = null;
	    stringTmp = System.getProperty("indigo.keyStorePrivateKeyPassword");
	    if (stringTmp != null) {
		keyStorePrivateKeyPassword = stringTmp;
	    }
	    
	    stringTmp = null;
	    stringTmp = System.getProperty("indigo.keyStoreType");
	    if (stringTmp != null) {
		keyStoreType = stringTmp;
	    }

	    stringTmp = null;
	    stringTmp = System.getProperty("indigo.keyManagerFactoryAlgorithm");
	    if (stringTmp != null) {
		keyManagerFactoryAlgorithm = stringTmp;
	    }
	    
	    stringTmp = null;
	    stringTmp = System.getProperty("indigo.trustStore");
	    if (stringTmp != null) {
		trustStore = stringTmp;
	    }
	    
	    stringTmp = null;
	    stringTmp = System.getProperty("indigo.trustStorePassword");
	    if (stringTmp != null) {
		trustStorePassword = stringTmp;
	    }

	    stringTmp = null;
	    stringTmp = System.getProperty("indigo.trustStoreType");
	    if (stringTmp != null) {
		trustStoreType = stringTmp;
	    }
	    
	    stringTmp = null;
	    stringTmp = System.getProperty("indigo.trustManagerFactoryAlgorithm");
	    if (stringTmp != null) {
		trustManagerFactoryAlgorithm = stringTmp;
	    }
	    
	    stringTmp = null;
	    stringTmp = System.getProperty("indigo.httpsProtocols");
	    if (stringTmp != null) {
		httpsProtocols = stringTmp;
	    }
	    
	    stringTmp = null;
	    stringTmp = System.getProperty("indigo.superUserName");
	    if (stringTmp != null) {
		superUserName = stringTmp;
	    }
	    
	    stringTmp = null;
	    stringTmp = System.getProperty("indigo.superUserPassword");
	    if (stringTmp != null) {
		superUserPassword = stringTmp;
	    }
	    
	    stringTmp = null;
	    stringTmp = System.getProperty("indigo.needMethodVerify");
	    if (stringTmp != null) {
		needMethodVerifyStr = stringTmp;
	    }
	    if (needMethodVerifyStr.equalsIgnoreCase("true")) {
		needMethodVerify = true;
	    }
	    else if (needMethodVerifyStr.equalsIgnoreCase("false")) {
		needMethodVerify  = false;
	    }
	    
	    // Die Auswertung von ClientAuth: 
	    // Ob eine Client-Authentication statt finden mu (true, maybe, false)
	    stringTmp = null;
	    stringTmp = System.getProperty("indigo.needClientAuth");
	    if (stringTmp != null) {
		clientAuth = stringTmp;
	    }
	    // Die Auswertung von ACCESS-MODE: 
	    if (clientAuth.equalsIgnoreCase("true")) {
		needClientAuth = true;
		wantClientAuth = false;
		
		stringTmp = null;
		stringTmp = System.getProperty("indigo.trueAccesscontrolllist");
		if (stringTmp != null) {
		    trueAccesscontrolllist = stringTmp;
		}
		currentAccesscontrolllist = trueAccesscontrolllist;
	    } else if (clientAuth.equalsIgnoreCase("maybe")) {
		needClientAuth = false;
		wantClientAuth = true;
		
		stringTmp = null;
		stringTmp = System.getProperty("indigo.maybeAccesscontrolllist");
		if (stringTmp != null) {
		    maybeAccesscontrolllist = stringTmp;
		}
		currentAccesscontrolllist = maybeAccesscontrolllist;
	    } else if (clientAuth.equalsIgnoreCase("false")) {
		needClientAuth = false;
		wantClientAuth = false;
		
		stringTmp = null;
		stringTmp = System.getProperty("indigo.falseAccesscontrolllist");
		if (stringTmp != null) {
		    falseAccesscontrolllist = stringTmp;
		}
		currentAccesscontrolllist = falseAccesscontrolllist;
	    }
	    
	    checkCurrentAccesscontrolllistSyntax(currentAccesscontrolllist);
	}
	else {
	    keyStore = "No need for a keystore";
	    keyStorePassword = "Nihil";
	    keyStorePrivateKeyPassword = "Nihil";
	    keyStoreType = "Nihil";
	    keyManagerFactoryAlgorithm = "Nihil";
	    trustStore = "No need for a truststore";
	    trustStorePassword = "Nihil";
	    trustStoreType = "Nihil";
	    trustManagerFactoryAlgorithm = "Nihil";
	    httpsProtocols = "Nihil";
	    superUserName = "Nihil";
	    superUserPassword = "Nihil";
	    needMethodVerifyStr = "false";
	    needMethodVerify = false;
	    clientAuth = "false";
	    needClientAuth = false;
	    wantClientAuth = false;
	    trueAccesscontrolllist = "(all)(all)(all)";
	    maybeAccesscontrolllist = "(all)(all)(all)";
	    falseAccesscontrolllist = "(all)(all)(all)";
	    currentAccesscontrolllist = falseAccesscontrolllist;
	}
	//                                                                         //
	/////////////////////////////////////////////////////////////////////////////
    }

    public static void init() {
    } 

    private static void checkCurrentAccesscontrolllistSyntax(String currentAM) {
 	String match = "(\\((\\w+\\,)*\\w+\\)){3}";
		    
  	if (currentAM.matches(match)) {
  	    return;
  	}
  	else {
  	    System.err.println("");
  	    System.err.println("The current access mode \"" + 
  			       currentAM + 
  			       "\" has syntax error!");
  	    System.exit(1);
  	}
    }

    public static void printProperty() {
	System.out.println("----------------------------------------------------------");
	System.out.println("Config File:                " + cf);
	System.out.println("DocumentRoot:               " + documentRoot);
	System.out.println("IndigoDir:                  " + indigoDir);
	System.out.println("Hostname:                   " + hostname);
	System.out.println("Hostaddress:                " + hostaddress);
	System.out.println("Server Port:                " + port);
	System.out.println("Server Timeout:             " + timeout);
	System.out.println("Provided Packages:          " + packages);
	System.out.println("Verbose Mode:               " + verboseModeStr);
	System.out.println("----------------------------------------------------------");
	System.out.println("The Used Protocol:          " + httpsProtocols);
	System.out.println("Verifying Methods:          " + needMethodVerify);
	System.out.println("Client Authentication:      " + clientAuth);
	System.out.println("Current Accesscontrolllist: " + currentAccesscontrolllist);
	System.out.println("----------------------------------------------------------");
	System.out.println("Keystore:                   " + keyStore);
	System.out.println("Keystore Type:              " + keyStoreType);
	System.out.println("Keystore Algorithm:         " + keyManagerFactoryAlgorithm);
	System.out.println("Truststore:                 " + trustStore);
	System.out.println("Truststore Type:            " + trustStoreType);
	System.out.println("Truststore Algorithm:       " + trustManagerFactoryAlgorithm);
	System.out.println("----------------------------------------------------------");
    }

    public  static String configFileToString() {
	return(cf);
    }

    public  static String documentRootToString() {
	return(documentRoot);
    }

    public  static String indigoDirToString() {
	return(indigoDir);
    }

    public  static String portToString() {
	return(port);
    }

    public  static int    portInt() {
	return(portInt);
    }

    public  static String timeoutToString() {
	return(timeout);
    }

    public  static int    timeoutInt() {
	return(timeoutInt);
    }

    public  static String hostnameToString() {
	return(hostname);
    }

    public  static String hostaddressToString() {
	return(hostaddress);
    }

    public  static String packagesToString() {
	return(packages);
    }

    public  static boolean secureExtension() {
	return(secureExtension);
    }

    public  static String keyStore() {
	return(keyStore);
    }
    
    public  static String keyStorePassword() {
	return(keyStorePassword);
    }

    public  static String keyStorePrivateKeyPassword() {
	return(keyStorePrivateKeyPassword);
    }

    public  static String keyStoreType() {
	return(keyStoreType);
    }

    public  static String keyManagerFactoryAlgorithm() {
	return(keyManagerFactoryAlgorithm);
    }

    public  static String trustStore() {
	return(trustStore);
    }
    
    public  static String trustStorePassword() {
	return(trustStorePassword);
    }
    
    public  static String trustStoreType() {
	return(trustStoreType);
    }
    
    public  static String trustManagerFactoryAlgorithm() {
	return(trustManagerFactoryAlgorithm);
    }
    
    public  static String httpsProtocols() {
	return(httpsProtocols);
    }
    
    public  static boolean needMethodVerify() {
	return(needMethodVerify);
    }
    
    public  static boolean verboseMode() {
	return(verboseMode);
    }

    public  static boolean needClientAuth() {
	return(needClientAuth);
    }

    public  static boolean wantClientAuth() {
	return(wantClientAuth);
    }

    public  static void clientAuth(SSLServerSocket netSSLServerSocket) {
	if (clientAuth.equalsIgnoreCase("true")) {
	    netSSLServerSocket.setNeedClientAuth(true);
	} else if (clientAuth.equalsIgnoreCase("maybe")) {
	    netSSLServerSocket.setWantClientAuth(true);
	} else if (clientAuth.equalsIgnoreCase("false")) {
	    netSSLServerSocket.setNeedClientAuth(false);
	}
    }

    public  static String getCurrentRunningMode() {
	return(clientAuth);
    }

    public  static String getCurrentAccesscontrolllist() {
	return(currentAccesscontrolllist);
    }

    protected static String getSuperUserAccount() {
	String superUserAccount;
	superUserAccount = superUserName + ":" + superUserPassword;
	return(superUserAccount);
    }

    protected static String getSuperUserAccountBase64() {
	String superUserAccount64;
//  	superUserAccount64 = new 
//  	    sun.misc.BASE64Encoder().encode((getSuperUserAccount()).getBytes());
	superUserAccount64 = Base64Converter.encode((getSuperUserAccount()).getBytes());
	return(superUserAccount64);
    }
    
} // IndigoConfig

