/*
 *  IndigoConnectionHandler.java  $Revision: 2.8 $
 *
 *  $Id: IndigoConnectionHandler.java,v 2.8 1999/12/22 11:03:44 cristian Exp $
 *  $Date: 1999/12/22 11:03:44 $
 *
 *  Copyright (C) 1999  Christian Moench
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Christian Moench
 *  Dept. of Computer Science
 *  Johann Wolfgang Goethe-University
 *  60325 Frankfurt, Germany
 *  Tel.  +49 69 79828358
 *  Fax.  +49 69 79823340
 *
 *  $Log: IndigoConnectionHandler.java,v $
 *  Revision 2.8  1999/12/22 11:03:44  cristian
 *  added group handling
 *
 *  Revision 2.7  1999/12/21 14:53:33  cristian
 *  added multiplexed standard output for document methods
 *
 *  Revision 2.6  1999/12/19 16:14:52  razi
 *  Including IndigoConfig.java
 *
 *  Revision 2.3  1999/12/19 15:59:00  razi
 *  IndigoConfig.java was added.
 *
 *  Revision 2.2  1999/12/15 16:37:21  cristian
 *  cleanups
 *
 *  Revision 1.8  1999/12/15 10:21:45  cristian
 *  Security now based on thread groups, all untrusted groups are restricted,
 *  this supports for multiple threads in document methods
 *
 *  Revision 1.7  1999/12/13 21:13:10  cristian
 *  added support for zip-files containing multiple classes
 *
 *  Revision 1.6  1999/12/13 18:30:06  cristian
 *  new method scheme started
 *
 *  Revision 1.5  1999/08/05 13:11:28  cristian
 *  private thread for methods, improved security handler,
 *  single start file for applet and application
 *
 *
 */
import java.io.*;
import java.net.*;
import java.lang.*;
import javax.net.*;
import javax.net.ssl.*;
import java.security.cert.*;
import java.lang.reflect.*;
import java.util.*;
import javax.security.auth.*;

class IndigoConnectionHandler extends Thread {

    protected MetaDocumentStore theStore;

    protected Socket      theClient;
    protected BoundaryReader input;
    protected PrintStream    output;

    protected String         method;
    protected String         path;

    protected String         protocol;
    protected String         server;
    protected String         authenticateRealm;
    
    protected Header         theHeader;

    protected static Hashtable messages;

    protected Thread acceptThread;
    
    private   boolean clientHasValidCert;
    private   IndigoAuthorization clientAuthorization;
    private   String  errorHeaderMessage;



    public IndigoConnectionHandler(Thread aThread, ThreadGroup systemGroup,
				 Socket aSocket,
				 MetaDocumentStore aStore)
	throws IOException {

	// Initialize the handler thread
	super(systemGroup, null, "Indigo handler thread for: "
	      + aSocket.getInetAddress().getHostAddress());

	// Remember the accept thread
	acceptThread = aThread ;

	// Remember the document store
	theStore = aStore;

	// Remember socket, input and output stream
	theClient = aSocket;
	input  = new BoundaryReader(
	        new InputStreamReader(theClient.getInputStream()));
	output = new PrintStream(theClient.getOutputStream());

	// Initialize some HTTP stuff
	messages = new Hashtable();
	messages.put(new Integer(200), "Ok");
	messages.put(new Integer(400), "Bad request");
	messages.put(new Integer(401), "Unauthorized");
	messages.put(new Integer(404), "Not found");
	messages.put(new Integer(405), "Access not allowed");
	messages.put(new Integer(500), "Internal error");

	protocol = "HTTP/1.0";
	server   = "Java-Indigo Server V1.9 with Secure Extensions";
	authenticateRealm = 
	    "Basic realm=\"Welcome To INDIGO - The Home Of Documents\"";

	return;
    }


    public void sendResponse(int code) {
	Integer key;

	key = new Integer(code);
	if (messages.containsKey(key)) {
	    output.println(protocol + " " + Integer.toString(code)
			   + " " + (String)messages.get(key));
	} else {
	    output.println(protocol + " " + Integer.toString(code));
	}
	sendHeader("Server", server);
	sendHeader("Date",  new Date().toString());
	return;
    }


    public void sendHeader(String name, String value) {
	output.println(name + ": " + value);
	output.flush();
    }


    public void endHeaders() {
	output.println();
	output.flush();
    }


    public boolean readRequest() {
	String   line;
	String[] words;


	// Read request from socket
  	try {
  	    line = input.readLine();
  	}
  	catch (IOException e) {
  	    sendResponse(500);
  	    sendHeader("Content-type", "text/plain");
  	    endHeaders();
  	    output.println(e.toString());
//    	    e.printStackTrace(System.err);
  	    return false;
  	}

	if (line == null) {
	    System.err.println("Connection closed by foreign host");
	    return false;
	}
	
	words = Utility.split(line);

	if (words.length != 3) {
	    sendResponse(400);
	    sendHeader("Content-type", "text/plain");
	    endHeaders();
	    output.println("bad request " + line);
	    return false;
	}

	method   = words[0];
	path     = words[1];
	protocol = words[2];
	
	// Diese Zeilen sind sinnvoll, wenn der Server mit Socket laeuft, 
	// aber die Clients mit SSL anfragen.
	if (!(method.equalsIgnoreCase("POST")) && 
	    !(method.equalsIgnoreCase("GET"))) {
	    sendResponse(400);
	    sendHeader("Content-type", "text/plain");
	    endHeaders();
	    output.println("Unknown method: " + method);
	    return false;
	}
	
	IndigoOut.errPrintln(line);
	return true;
    }


    /*
     * Handle one incoming request.
     */
    public void run() {
	boolean sSLHandshakeException = false;
	
	if (IndigoConfig.secureExtension()) {
	    try {
		SSLSession sslSession = ((SSLSocket)theClient).getSession();
		Certificate[] peerCerts = sslSession.getPeerCertificates();
		X509Certificate clientCert = (X509Certificate)peerCerts[0];
		
		java.security.Principal clientSubjectDN = clientCert.getSubjectDN();
		java.security.Principal clientIssuerDN  = clientCert.getIssuerDN();
		Subject subject = new Subject();
		subject.getPrincipals().add(clientSubjectDN);
		subject.getPrincipals().add(clientIssuerDN);

		clientCert.checkValidity();
		clientHasValidCert = true;
		
		IndigoOut.errPrintln(" ");
		IndigoOut.errPrintln(" ");
		IndigoOut.errPrintln("IndigoConnectionHandler:");
		IndigoOut.errPrintln("    Client-Subject:  " + clientCert.getSubjectDN());
		IndigoOut.errPrintln("    Client-Issuer :  " + clientCert.getIssuerDN());
		IndigoOut.errPrintln(" ");
		
//  	    for (int i = 0; i < peerCerts.length; i++) {
//    		IndigoOut.errPrintln("IndigoConnectionHandler.java: Certificate: " 
//  				   + peerCerts[i].getType() + "  " 
//	                           + peerCerts[i].toString());
//  	    }
//  	    IndigoOut.errPrintln("IndigoConnectionHandler.java: PeerHostname: " 
//  			       + sslSession.getPeerHost());
//  	    subject.getPublicCredentials().add(credential);
//  	    Certificate[] mycerts = sslSession.getLocalCertificates();
	    }
	    catch (SSLPeerUnverifiedException e) {
		IndigoOut.errPrintln(
		    "IndigoConnectionHandler: Client has not any certificate!");
		errorHeaderMessage = "You have not any certificate!";
		clientHasValidCert = false;
	    }
	    catch (CertificateExpiredException e) {
		IndigoOut.errPrintln(
		    "IndigoConnectionHandler: Client's certificate has expired!");
		errorHeaderMessage = "Your certificate has expired!";
		clientHasValidCert = false;
	    }
	    catch (CertificateNotYetValidException e) {
		IndigoOut.errPrintln(
		    "IndigoConnectionHandler: Client's certificate is not yet valid!");
		errorHeaderMessage = "Your certificate is not yet valid!"; 
		clientHasValidCert = false;
	    }
//  	    catch (SSLHandshakeException e) {
//  		IndigoOut.errPrintln(
//  		    "IndigoConnectionHandler: Handshake Problems!");
//  		errorHeaderMessage = "Handshake Problems!"; 
//  		clientHasValidCert = false;
//  		sSLHandshakeException = true;
//  	    } 
	
	
	    // If there is problem with the client-cert and the 
	    // client-authentification is needed, close the socket.
	    if (IndigoConfig.secureExtension() && 
		(clientHasValidCert==false) && 
		(IndigoConfig.needClientAuth())) {
		if (!sSLHandshakeException) {
		    try {
			sendResponse(401);
			sendHeader("Content-type", "text/plain");
			endHeaders();
			output.println(errorHeaderMessage);
			output.flush();
		    }
		    catch (Exception e) {
			IndigoOut.errPrintln(e.toString());
			e.printStackTrace(System.err);
		    }
		}
		try {
		    theClient.close();
		}
		catch (IOException e) {
		    IndigoOut.errPrintln(e.toString());
		    e.printStackTrace(System.err);
		}
		return;
	    }
	}

	// Die erste Requestzeile wird behandelt!
	if (readRequest() == false) {
	    try {
		output.flush();
		theClient.close();
	    }
	    catch (IOException e) {
		IndigoOut.errPrintln(e.toString());
		e.printStackTrace(System.err);
	    }
	    return;
	}
	
	LocalURL localSpec;
	try {	
	    theHeader = new Header(input);
	    localSpec = new LocalURL(path);
	    
	    clientAuthorization = new 
		IndigoAuthorization(clientHasValidCert,
				    (theHeader.getAuthorizationValueBASE64()));
	    
	    if (method.equals("GET")) {
 		handleGet(localSpec);
	    } else if (method.equals("POST")) {
		handlePost(localSpec);
	    } else {
		sendResponse(400);
		sendHeader("Content-type", "text/plain");
		endHeaders();
		output.println("Unknown method: " + method);
	    }
	}
	catch (Exception e) {
	    sendResponse(401);
	    sendHeader("Content-type", "text/plain");
	    endHeaders();
	    output.println(e.toString());
	    e.printStackTrace(System.err);
	}

	try {
	    output.flush();
	    theClient.close();
	}
	catch (IOException e) {
	    output.println(e.toString());
	    IndigoOut.errPrintln(e.toString());
	    e.printStackTrace(System.err);
	}
	return;
    }


    public void handleGet(LocalURL localSpec)
//  	    throws IOException, SecurityException {
	    throws IOException {

	if (localSpec.thePath.startsWith("/diglib/invoke/")) {

	    // Perform invoke on the document
	    if (localSpec.thePathParts.length < 3) {
		sendResponse(400);
		sendHeader("Content-type", "text/plain");
		endHeaders();
		output.println("Document id not found in request");
		return;
	    }

	    String docid = localSpec.thePathParts[2];
	    if (localSpec.theSearchParts.length < 1) {
		sendResponse(400);
		sendHeader("Content-type", "text/plain");
		endHeaders();
		output.println("No method specified");
		return;
	    }

	    String method = localSpec.theSearchParts[0];
	    if (clientAuthorization.authorizedFor("invoke")) {
		handleInvoke(docid, method, localSpec.theSearchParts);
	    }
	    else {
		sendResponse(401);
		sendHeader("Content-type", "text/plain");
		if (clientAuthorization.needWWWAuthenticateResponseHeader()) {
		    sendHeader("WWW-Authenticate", authenticateRealm);
		}
		endHeaders();
		output.println("Not Authorized for this method: invoke");
	    }
	    return;

	} else if (localSpec.thePath.startsWith("/diglib/attributes/")) {
	    if (localSpec.thePathParts.length < 3) {
		sendResponse(400);
		sendHeader("Content-type", "text/plain");
		endHeaders();
		output.println("Document id not found in request");
		return;
	    }
	    if (clientAuthorization.authorizedFor("attributes")) {
		getAttributes(localSpec.thePathParts[2]);
	    }
	    else {
		sendResponse(401);
		sendHeader("Content-type", "text/plain");
		if (clientAuthorization.needWWWAuthenticateResponseHeader()) {
		    sendHeader("WWW-Authenticate", authenticateRealm);
		}
		endHeaders();
		output.println("Not Authorized for this method: attributes");
	    }
	    return;
	} else if (localSpec.thePath.startsWith("/diglib/documents")) {

	    String[] theDocuments = theStore.getDocuments();

	    if (clientAuthorization.authorizedFor("documents")) {
		sendResponse(200);
		sendHeader("Content-type", "text/plain");
		endHeaders();
		output.println("documents:");
		for (int i = 0; i < theDocuments.length; i++) {
		    output.println(theDocuments[i]);
		}
		output.println();
	    }
	    else {
		sendResponse(401);
		sendHeader("Content-type", "text/plain");
		if (clientAuthorization.needWWWAuthenticateResponseHeader()) {
		    sendHeader("WWW-Authenticate", authenticateRealm);
		}
		endHeaders();
		output.println("Not Authorized for this method: documents");
	    }
	    return;
	} else if (localSpec.thePath.startsWith("/diglib/quit")) {
	    if (clientAuthorization.authorizedFor("quit")) {
		sendResponse(200);
		sendHeader("Content-type", "text/plain");
		endHeaders();
		output.println("Trying to quit.");
		acceptThread.interrupt();
	    }
	    else {
		sendResponse(401);
		sendHeader("Content-type", "text/plain");
		if (clientAuthorization.needWWWAuthenticateResponseHeader()) {
		    sendHeader("WWW-Authenticate", authenticateRealm);
		}
		endHeaders();
		output.println("Not Authorized for this method: quit");
	    }
	    return;
        } else if (localSpec.thePath.startsWith("/diglib/packages")) {
	    if (clientAuthorization.authorizedFor("packages")) {
		sendResponse(200);
		sendHeader("Content-type", "text/plain");
		endHeaders();
		output.println
		    ("Provided packages: " + IndigoConfig.packagesToString() + "\n");
	    }
	    else {
		sendResponse(401);
		sendHeader("Content-type", "text/plain");
		if (clientAuthorization.needWWWAuthenticateResponseHeader()) {
		    sendHeader("WWW-Authenticate", authenticateRealm);
		}
		endHeaders();
		output.println("Not Authorized for this method: packages");
	    }
            return;
 	}
	sendResponse(404);
	sendHeader("Content-type", "text/plain");
	endHeaders();
	output.println("The resource " + path + " was not found");
	return;
    }


    public void handlePost(LocalURL localSpec) {

	if (localSpec.thePath.equals("/diglib/deliver")) {
	    if (clientAuthorization.authorizedFor("deliver")) {

		// Get new id and store document
		String id = Id.getId();
		if (theStore.decode(id, input) == false) {
		    sendResponse(400);
		    sendHeader("Content-type", "text/plain");
		    endHeaders();
		    output.println(theStore.getErrorMessage());		
		    return;
		}
		
		// Document successfuly stored, send result message
		sendResponse(200);
		sendHeader("Content-type", "text/plain");
		endHeaders();
		output.println("document-id: " + id);
	    }
	    else {
		sendResponse(401);
		sendHeader("Content-type", "text/plain");
		if (clientAuthorization.needWWWAuthenticateResponseHeader()) {
		    sendHeader("WWW-Authenticate", authenticateRealm);
		}
		endHeaders();
		output.println("Not Authorized for this method: deliver");
	    }
	    return;
	}

	sendResponse(405);
	sendHeader("Content-type", "text/plain");
	endHeaders();
	output.println("Posting to " + path + " not allowed");
    }


    public void getAttributes(String id) {

	if (id.length() == 0) {
	    sendResponse(400);
	    sendHeader("Content-type", "text/plain");
	    endHeaders();
	    output.println("No document identifier specified");
	    return;
	}

	Attributes theAttributes;
	try {
	    theAttributes = theStore.getAttributes(id);
	}
	catch (IOException e) {
	    sendResponse(404);
	    sendHeader("Content-type", "text/plain");
	    endHeaders();
	    output.println(e.getMessage());
	    return;
	}
	sendResponse(200);
	sendHeader("Content-type", "text/plain");
	endHeaders();
	output.println("attributes:");

	Enumeration attributeNames;
	attributeNames = theAttributes.theAttributes.keys();
	while (attributeNames.hasMoreElements()) {
	    String name, value;
	    try {
		name  = (String)attributeNames.nextElement();
		value = (String)theAttributes.theAttributes.get(name);
		output.println(name + " " + value);
	    }
	    catch (NoSuchElementException e) {
		e.printStackTrace(System.err);
		return;
	    }
	}
	output.println();
    }


    public void handleInvoke(String docid, String method, String[] args) {

	Methods theMethods;
	try {
	    theMethods = theStore.getMethods(docid);
	}
	catch (IOException e) {
	    sendResponse(404);
	    sendHeader("Content-type", "text/plain");
	    endHeaders();
	    output.println(e.getMessage());
	    return;
	}
	
	URL methodLocation;
	methodLocation = (URL)theMethods.theMethods.get(method);
	if (methodLocation == null) {
	    sendResponse(404);
	    sendHeader("Content-type", "text/plain");
	    endHeaders();
	    output.println("Method " + method + " not found in document "
			   + docid);
	    return;
	}

	Class aClass;
	MethodLoader aLoader = new MethodLoader(theStore, docid);
	try {
	    aClass = aLoader.loadClass(method);
	}
	catch (SecurityException e) {
	    sendResponse(500);
	    sendHeader("Content-type", "text/plain");
	    endHeaders();
	    output.println("Security error!");
	    output.println("This method is not trustworthy!");
	    output.println(e.toString());
//  	    e.printStackTrace(System.err);
	    IndigoOut.errPrintln("This method is not trustworthy!");
	    return;
	}	
	catch (ClassNotFoundException e) {
	    sendResponse(500);
	    sendHeader("Content-type", "text/plain");
	    endHeaders();
	    output.println("Could not load all classes for " + method);
	    output.println(e.toString());
//  	    e.printStackTrace(System.err);
	    IndigoOut.errPrintln("Could not load all classes for " + method);
	    return;
	}

	Method mainMethod;
	Class[] mainArgs= new Class[1];
	try {
	    Object a = Array.newInstance(
	            Class.forName("java.lang.String"), 1);
	    mainArgs[0] = a.getClass();
	    mainMethod = aClass.getMethod("main", mainArgs);
	}
	catch (SecurityException e) {
	    sendResponse(500);
	    sendHeader("Content-type", "text/plain");
	    endHeaders();
	    output.println("Security ERROR!");
	    output.println("This method is not trustworthy!");
	    output.println(e.toString());
	    e.printStackTrace(System.err);
	    return;
	}
	catch (Exception e) {
	    sendResponse(500);
	    sendHeader("Content-type", "text/plain");
	    endHeaders();
	    output.println("Could not find main method in class " + method);
	    output.println(e.toString());
	    e.printStackTrace(System.err);
	    return;
	}

	// The server command was a success
	sendResponse(200);
	sendHeader("Content-type", "text/plain");
	endHeaders();

	// Invoke the static main method
	try {

	    /*-
	     * Create arguments for the method:
	     *
	     * arg[0]: id
	     * arg[1]: method-argument 1
	     *           .
	     *           .
	     *           .
	     *
	     * arg[n]: method-argument n
	     */
	    Object[] theArguments = new Object[1];
	    theArguments[0] = new String[args.length];
	    ((String[])theArguments[0])[0] = docid;
	    for (int i = 1; i < args.length; i++) {
		((String[])theArguments[0])[i] = args[i];
	    }

	    // Create a handler thread for the method
	    MethodThread aThread;
	    ThreadGroup methodGroup;

	    methodGroup = new ThreadGroup(docid + "#" + method + "-group");
	    aThread = new MethodThread
		(methodGroup,
		 docid, method, mainMethod, theArguments,
		 theStore);
	    aThread.clientSocket = theClient;

	    /*
	     * If we are no applet, our own security manager is
	     * running. Add the document method thread to the set of
	     * supervised threads.
	     */
	    if (Indigo.isApplet == false) {
		IndigoSecurityManager aManager;
		aManager = (IndigoSecurityManager)System.getSecurityManager();
		aManager.restrictTo(methodGroup, docid);
	    }

	    // Redirect threads System.out calls to output.
	    ((DispatchedStream)System.out).addGroupOutput(methodGroup, output);

	    // Store main thread in Self-package
	    Self.theMethodThreads.put(methodGroup, aThread);

	    // Start the thread and wait until it returns.
	    aThread.start();
	    aThread.join();


	    // Remove main thread from Self-package
	    Self.theMethodThreads.remove(methodGroup);

	    // Restore the standard output.
	    ((DispatchedStream)System.out).removeGroupOutput(methodGroup);

	    // The thread returned, destroy the group and all its
	    // threads.
	    try {
		methodGroup.destroy();
	    }
	    catch (IllegalThreadStateException e) {
	    }

	    /*
	     * If our own security manager is running, remove the
	     * document method thread to the set of supervised
	     * threads.
	     */
	    if (Indigo.isApplet == false) {
		IndigoSecurityManager aManager;
		aManager = (IndigoSecurityManager)System.getSecurityManager();
		aManager.unrestrict(methodGroup);
	    }

	    if (aThread.getException() != null) {
		output.println("Exception " 
			       + aThread.getException().toString()
			       + "while executing " + method);
		aThread.getException().printStackTrace(System.err);
	    }
	}
	catch (Exception e) {
	    output.println(e.toString());
	    e.printStackTrace(System.err);
	}
	return;
    }
}
