/**
 * IndigoSSLContext.java
 *
 *
 * Created: Sat Feb  9 15:42:08 2002
 *
 * @author Razi Lotfi-Tabrizi
 * @version
 */

import java.io.*;
import java.net.*;
import java.security.KeyStore;
import javax.net.*;
import javax.net.ssl.*;
import javax.security.cert.X509Certificate;
import java.lang.*;
import java.util.*;


public class IndigoSSLContext {
    
    protected  SSLContext sslctx;
    protected  KeyManagerFactory kmf;
    protected  TrustManagerFactory tmf;
    protected  KeyStore ks;    // KeyManager-Store
    protected  KeyStore ts;    // TrustManager-Store   
    
    public IndigoSSLContext() {
	// set up key manager to do server authentication
	try {
            //SSL or TLS
	    sslctx = SSLContext.getInstance(IndigoConfig.httpsProtocols()); 

	    // KeyStore initialisieren und lesen:
	    char[] passphrase = 
		(IndigoConfig.keyStorePassword()).toCharArray();
	    char[] passphrasePK = 
		(IndigoConfig.keyStorePrivateKeyPassword()).toCharArray();
	    
	    kmf = KeyManagerFactory.
		getInstance(IndigoConfig.keyManagerFactoryAlgorithm());
	    ks = KeyStore.
		getInstance(IndigoConfig.keyStoreType());
	    ks.load(new FileInputStream(IndigoConfig.keyStore()), passphrase);
	    kmf.init(ks, passphrasePK);
	    
	    // TrustStore initialisieren und lesen:
	    char[] passphrase1 = (IndigoConfig.trustStorePassword()).toCharArray();
	    tmf = TrustManagerFactory.
		getInstance(IndigoConfig.trustManagerFactoryAlgorithm());
	    ts = KeyStore.getInstance(IndigoConfig.trustStoreType());
	    ts.load(new FileInputStream(IndigoConfig.trustStore()), passphrase1);
	    tmf.init(ts);
	    
	    sslctx.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);
	} 
	catch (Exception e) {
	    e.printStackTrace();
	}
    }

    public SSLContext getSSLContext() {
	return(sslctx);
    }

    public SSLSocketFactory getSSLSocketFactory() {
	return((SSLSocketFactory)((this.sslctx).getSocketFactory()));
    }

    public SSLServerSocketFactory getSSLServerSocketFactory() {
	return((SSLServerSocketFactory)((this.sslctx).getServerSocketFactory()));
    }

    public static SSLSocketFactory getStaticSSLSocketFactory() {
	IndigoSSLContext isc = new IndigoSSLContext();
	SSLSocketFactory sssf = 
	    (SSLSocketFactory)((isc.getSSLContext()).getSocketFactory());
	return(sssf);
    }

    public static SSLServerSocketFactory getStaticSSLServerSocketFactory() {
	IndigoSSLContext isc        = new IndigoSSLContext();
	SSLServerSocketFactory sssf = 
	    (SSLServerSocketFactory)((isc.getSSLContext()).getServerSocketFactory());
	return(sssf);
    }

} // IndigoSSLContext





