/*
 *  SecurityManger.java  $Revision: 2.6 $
 *
 *  $Id: IndigoSecurityManager.java,v 2.6 1999/12/19 16:14:53 razi Exp $
 *  $Date: 1999/12/19 16:14:53 $ $Author: razi $
 *
 *  Copyright (C) 1999  Christian Moench
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Christian Moench
 *  Dept. of Computer Science
 *  Johann Wolfgang Goethe-University
 *  60325 Frankfurt, Germany
 *  Tel.  +49 69 79828358
 *  Fax.  +49 69 79823340
 *
 *  $Log: IndigoSecurityManager.java,v $
 *  Revision 2.6  1999/12/19 16:14:53  razi
 *  Including IndigoConfig.java
 *
 *  Revision 2.3  1999/12/15 22:29:40  cristian
 *  removed debug info, added exception handling
 *
 *  Revision 2.2  1999/12/15 16:37:21  cristian
 *  cleanups
 *
 *  Revision 1.5  1999/12/15 16:27:57  cristian
 *  cleanups
 *
 *  Revision 1.4  1999/12/15 10:21:45  cristian
 *  Security now based on thread groups, all untrusted groups are restricted,
 *  this supports for multiple threads in document methods
 *
 *  Revision 1.3  1999/12/13 18:30:06  cristian
 *  new method scheme started
 *
 *  Revision 1.2  1999/08/05 13:11:29  cristian
 *  private thread for methods, improved security handler,
 *  single start file for applet and application
 *
 *  Revision 1.1.1.1  1999/07/28 15:55:58  cristian
 *  Initial import
 *
 *
 */
import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;


public final class IndigoSecurityManager extends SecurityManager {

    // These thread groups are restricted to handle certain documents
    private Hashtable restrictedGroups;
    private String    documentRoot;


    // Check wether `aGroup' or one of its parents is restricted.
    private boolean isRestricted(ThreadGroup aGroup) {
	while (aGroup != null) {
	    if (restrictedGroups.containsKey(aGroup)) {
		return true;
	    }
	    aGroup = aGroup.getParent();
	}
	return false;
    }


    // Check wether the group if the current thread ore of its parents
    // is restricted.
    private boolean isCurrentRestricted() {
	return isRestricted(Thread.currentThread().getThreadGroup());
    }


    /*
     * Create a Indigo security manager
     */
    public IndigoSecurityManager(String root) {
	super();
	documentRoot = root;
	restrictedGroups = new Hashtable();
    }


    /*
     * Restrict a thread group to handle a certain document id.
     */
    public void restrictTo(ThreadGroup aGroup, String id) {
	// TODO: this is repeated in MetaDocumentStore!
	String dir = documentRoot + File.separator + id;
	restrictedGroups.put(aGroup, dir);
    }


    /*
     * Unrestrict a thread group
     */
    public void unrestrict(ThreadGroup aGroup) {
	if (isCurrentRestricted() == true) {
	    throw new SecurityException
		("Not allowed to unrestrict group " + aGroup.getName());
	}
	restrictedGroups.remove(aGroup);
    }


    /*
     * The following operations are allowed to everybody.
     */

/*  Razi-Mod

    public void checkListen(int port) {
	return;
    }


    public void checkAccept(String host, int port) {
	return;
    }


    public void checkConnect(String host, int port) {
	return;
    }


    public void checkConnect(String host, int port, Object context) {
	return;
    }


    public void checkPackageAccess(String pkg) {
	return;
    }


    public void checkPackageDefinition(String pkg) {
	return;
    }


    public void checkMulticast(InetAddress x) {
	return;
    }


    public void checkMulticast(InetAddress x, byte y) {
	return;
    }


    public void checkMemberAccess(Class aClass, int aMember) {
	System.err.println("[disabled]checkMemberAccess(Class <"
			   + aClass.getName() + ">, int "
			   + Integer.toString(aMember));
	return;
    }


    public void checkAccess(Thread x) {
	System.err.println("[disabled]checkAccess(Thread <"
			   + x.getName() + ">)");
	return;
    }


    public void checkAccess(ThreadGroup x) {
	System.err.println("[disabled]checkAccess(ThreadGroup <"
			   + x.getName() + ">)");
	return;
    }
*/
    /*
     * The following operations are restricted in some way.
     */
/*
    public void checkCreateClassLoader() {
       	if (isCurrentRestricted()) {
	    throw new SecurityException("Not allowed to create class loader");
	}
	return;
    }


    public boolean checkTopLevelWindow(Object window) {
       	if (isCurrentRestricted()) {
	    System.err.println("[disabled]checkTopLevelWindow()");
	    return true;
	}
	return true;
    }


    public void checkLink(String lib) {
       	if (isCurrentRestricted()) {
	    if (lib.equals("awt")) {
		return;
	    }
	    System.err.println("checkLink(" + lib + ")");
	    throw new SecurityException("Not allowed to load library " + lib);
	}
	return;
    }


    public void checkRead(String file) throws SecurityException {
       	if (isCurrentRestricted()) {
	    String dir;
	    dir = (String)restrictedGroups.get
		(Thread.currentThread().getThreadGroup());
	    if (file.startsWith(dir)) {
		return;
	    }
	    if (file.endsWith(".properties")) {
		return;
	    }
	    System.err.println("checkRead(" + file + ")");
	    throw new SecurityException("Not allowed to read from " + file);
	}
	return;
    }


    public void checkRead(String file, Object x) throws SecurityException {
	checkRead(file);
    }


    public void checkRead(FileDescriptor fd) throws SecurityException {
       	if (isCurrentRestricted()) {
	    if (inClass("java.net.Socket")) {
		return;
	    }
	    throw new SecurityException("Not allowed to read from " + fd);
	}
	return;
    }


    public void checkWrite(FileDescriptor fd) throws SecurityException {
       	if (isCurrentRestricted()) {
	    if (inClass("java.net.Socket")) {
		return;
	    }
	    throw new SecurityException("Not allowed to write to " + fd);
	}
	return;
    }


    public void checkDelete(String file) throws SecurityException {
       	if (isCurrentRestricted()) {
	    String dir;

	    dir = (String)restrictedGroups.get
		(Thread.currentThread().getThreadGroup());
	    if (file.startsWith(dir)) {
		return;
	    }
	    throw new SecurityException("Not allowed to delete " + file);
	}
	return;
    }


    public void checkWrite(String file) throws SecurityException {
       	if (isCurrentRestricted()) {
	    String dir;

	    dir = (String)restrictedGroups.get
		(Thread.currentThread().getThreadGroup());
	    if (file.startsWith(dir)) {
		return;
	    }
	    throw new SecurityException("Not allowed to write to " + file);
	}
	return;
    }


    public void checkExit(int x) throws SecurityException {
       	if (isCurrentRestricted()) {
	    throw new SecurityException("Not allowed to call exit");
	}
	return;
    }


    public void checkPropertyAccess(String key) {
       	if (isCurrentRestricted()) {
	    System.err.println("[disabled]checkPropertyAccess(" + key + ")");
	    throw new SecurityException("Not allowed to access property "
	    			+ key);
	}
	return;
    }


    public void checkPropertiesAccess() {
       	if (isCurrentRestricted()) {
	    System.err.println("[disabled]checkPropertiesAccess()");
	    throw new SecurityException("Not allowed to access properties");
	}
	return;
    }
    */
}
