/*
 *  IndigoServer.java  $Revision: 2.9 $
 *
 *  $Id: IndigoServer.java,v 2.9 1999/12/22 11:03:44 cristian Exp $
 *  $Date: 1999/12/22 11:03:44 $
 *
 *  Copyright (C) 1999  Christian Moench
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Christian Moench
 *  Dept. of Computer Science
 *  Johann Wolfgang Goethe-University
 *  60325 Frankfurt, Germany
 *  Tel.  +49 69 79828358
 *  Fax.  +49 69 79823340
 *
 *  $Log: IndigoServer.java,v $
 *  Revision 2.9  1999/12/22 11:03:44  cristian
 *  added group handling
 *
 *  Revision 2.8  1999/12/21 14:53:34  cristian
 *  added multiplexed standard output for document methods
 *
 *  Revision 2.7  1999/12/20 20:02:16  razi
 *  Elimination of PortBug!
 *
 *  Revision 2.6  1999/12/19 16:14:53  razi
 *  Including IndigoConfig.java
 *
 *  Revision 2.4  1999/12/19 15:59:00  razi
 *  IndigoConfig.java was added.
 *
 *  Revision 2.2  1999/12/15 16:37:21  cristian
 *  cleanups
 *
 *  Revision 1.5  1999/12/15 10:21:46  cristian
 *  Security now based on thread groups, all untrusted groups are restricted,
 *  this supports for multiple threads in document methods
 *
 *  Revision 1.4  1999/12/13 18:30:07  cristian
 *  new method scheme started
 *
 *  Revision 1.3  1999/08/05 13:11:31  cristian
 *  private thread for methods, improved security handler,
 *  single start file for applet and application
 *
 *  Revision 1.2  1999/07/30 14:21:50  cristian
 *  removed IndigoApplet.java
 *  Indigo.java now serves as Applet and Application starting point
 *
 *
 */
import java.io.*;
import java.net.*;
import java.security.KeyStore;
import javax.net.*;
import javax.net.ssl.*;
import javax.security.cert.X509Certificate;
import java.lang.*;
import java.util.*;


public class IndigoServer {

    static public void startServing(String documentRoot) {
	IndigoConnectionHandler aHandler;
	ThreadGroup           systemGroup;
	int                   portNumber;

	// Create the document store.
	MetaDocumentStore aStore = new MetaDocumentStore(documentRoot);

	// Initialize packages
	Pool.init();
	Self.init();
	
	// Create an output stream for multiple threads
	System.setOut(new DispatchedStream(System.out));

	// Set our own security manager if we are no applet
	if (Indigo.isApplet == false) {
	    System.setSecurityManager(new IndigoSecurityManager(documentRoot));
	}

	try {
	    Thread.currentThread().setName("Indigo accept thread");
	    
	    // Create a thread group for system threads
	    systemGroup = new ThreadGroup("Indigo system group");
	    
	    // Get the port number
	    portNumber = IndigoConfig.portInt();
	    
	    // Create a server socket and start serving
  	    ServerSocket theServer = 
  		IndigoSSLServerSocket.getServerSocket(portNumber);
	    theServer.setSoTimeout(IndigoConfig.timeoutInt());

	    while (Thread.interrupted() == false) {
		Socket aClient;

		try {
		    aClient = theServer.accept();
		    aHandler = new IndigoConnectionHandler(
			Thread.currentThread(), systemGroup,
			aClient, aStore);
		    aHandler.start();
		}
//  		catch (InterruptedIOException e) {
  		catch (Exception e) {
		    if (e.getMessage().equals("operation interrupted")) {
			break;
		    }
		}
	    }

	    // Termination requested, wait for the remaining threads
	    System.err.println("Termination request, killing threads");
	    systemGroup.stop();
	}
	catch (Exception e) {
	    e.printStackTrace(System.err);
	}

	return;
    }
}
