/*
 *  MetaDocumentStore.java  $Revision: 2.6 $
 *
 *  $Date: 1999/12/19 16:14:54 $ $Author: razi $
 *
 *  Copyright (C) 1999  Christian Moench
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Christian Moench
 *  Dept. of Computer Science
 *  Johann Wolfgang Goethe-University
 *  60325 Frankfurt, Germany
 *  Tel.  +49 69 79828358
 *  Fax.  +49 69 79823340
 *
 *  $Log: MetaDocumentStore.java,v $
 *  Revision 2.6  1999/12/19 16:14:54  razi
 *  Including IndigoConfig.java
 *
 *  Revision 2.2  1999/12/15 16:37:23  cristian
 *  cleanups
 *
 *  Revision 1.4  1999/12/13 21:13:10  cristian
 *  added support for zip-files containing multiple classes
 *
 *  Revision 1.3  1999/12/13 18:30:08  cristian
 *  new method scheme started
 *
 *  Revision 1.2  1999/07/29 18:51:50  cristian
 *  Initial release
 *
 *  Revision 1.1.1.1  1999/07/28 15:55:58  cristian
 *  Initial import
 *
 *
 */
import java.io.*;
import java.net.*;
import javax.net.ssl.*;
import javax.security.cert.X509Certificate;
import java.lang.*;
import java.util.*;
import java.util.zip.*;
import java.util.jar.*;
import java.util.jar.Attributes.Name.*;
import java.security.cert.*;


public class MetaDocumentStore {

    protected String    documentRoot;
    protected String    errorMessage;
    protected boolean   methodFromHttpsAddress;

    public String getDocumentDir(String id) {
	return documentRoot + File.separator + id;
    }


    public String getDocumentRoot() {
	return documentRoot;
    }


    protected String getCodeDir(String id) {
	return getDocumentDir(id) + File.separator + "code";
    }


    /*
     *  Returns a File object referencing that points to the code
     *  referenced by `location'. Loads the code from the URL
     *  `location' if it is not already on the local disk.
     */
    protected void retrieveContent(URL location, String fileName)
	    throws IOException {

	// Load file from `location'
	InputStream   codeInput;
	URLConnection aConnection;
	
	// Server can download his documentmethods from a SSL-Server (https-protocol) 
	if ((IndigoConfig.secureExtension()) && 
	    (location.getProtocol().equalsIgnoreCase("https"))) {
	    this.methodFromHttpsAddress = true;
	    aConnection = location.openConnection();
	    ((HttpsURLConnection) aConnection).
		setSSLSocketFactory(IndigoSSLContext.getStaticSSLSocketFactory());
	}
	else {
	    this.methodFromHttpsAddress = false;
	    aConnection = location.openConnection();
	    aConnection.connect();
	}

	codeInput = aConnection.getInputStream();
	FileOutputStream fileOutput = new FileOutputStream(fileName);

	byte[] buffer = new byte[1024];
	int length = codeInput.read(buffer);
	while (length >= 0) {
	    fileOutput.write(buffer, 0, length);
	    length = codeInput.read(buffer);
	}
	codeInput.close();
	fileOutput.close();
    }


    public String getAttributesName(String id) {
	return getDocumentDir(id) + File.separator + "attributes";
    }


    public String getMethodsName(String id) {
	return getDocumentDir(id) + File.separator + "methods";
    }


    public String getContentName(String id) {
	return getDocumentDir(id) + File.separator + "content";
    }


    /*
     * Return the name of the file that contains the zip-archive of
     * the classes which are necessary for the method `method'.
     */
    public String getCodeName(String id, String method) {
	return getCodeDir(id) + File.separator + method;
    }


    public MetaDocumentStore(String aDirectory) throws SecurityException {
	File documentRootFile;

	/*
	 * Create the document root directory, which will hold all
	 * meta documents that are stored in this store.
	 */
	documentRoot = aDirectory;
	documentRootFile = new File(documentRoot);
	if (documentRootFile.exists() == false) {
	    documentRootFile.mkdirs();
	}

	errorMessage = "No error";
	return;
    }


    public String getErrorMessage() {
	return errorMessage;
    }


    /*
     * Create a list of all documents
     */
    public String[] getDocuments() throws IOException, SecurityException {
	return new File(documentRoot).list();
    }


    public boolean documentExists(String id)
	throws IOException, SecurityException {
	String[] theDocuments;

	theDocuments = getDocuments();
	for (int i = 0; i < theDocuments.length; i++) {
	    if (id.equals(theDocuments[i])) {
		return true;
	    }
	}
	return false;
    }


    /*
     * Retrieve the methods from disk
     */
    public Methods getMethods(String id) throws IOException {
	BufferedReader methodsReader;
	Methods        theMethods;

	if (new File(getDocumentDir(id)).exists() == false) {
	    throw new IOException("Document " + id + " not found");
	}
	theMethods = new Methods();
	methodsReader = new BufferedReader(
	        new FileReader(getMethodsName(id)));
        theMethods.readFrom(methodsReader);
	return theMethods;
    }


    /*
     * Retrieve the attributes from disk
     */
    public Attributes getAttributes(String id) throws IOException {
	BufferedReader attributesReader;
	Attributes     theAttributes;

	if (new File(getDocumentDir(id)).exists() == false) {
	    throw new IOException("Document " + id + " not found");
	}
	theAttributes = new Attributes();
	attributesReader = new BufferedReader(
	        new FileReader(getAttributesName(id)));
	theAttributes.readFrom(attributesReader);
	return theAttributes;
    }

    /*
     * Retrieve the AuthorCertificate from attribute file
     */
    public Certificate getAuthorCertificate(String id) {
	String multiLineBegin =  "<MULTILINE>";
	String multiLineEnd   = "</MULTILINE>";
	String rawCert;
	String authorCertStr = null;
	Certificate authorCert = null;
	Attributes authorAttri;
	CertificateFactory cf;
	
	try {
	    authorAttri = this.getAttributes(id);
	    rawCert = (String)(authorAttri.theAttributes.get("AuthorCertificate"));
	    if (rawCert == null) {
		throw new IOException("AuthorCertificate is null!");
	    }
	}
	catch (Exception e) {
	    return null;
	}	
	
	try {
	    java.util.regex.Pattern pattern =
		java.util.regex.Pattern.compile("(.+)(-----BEGIN CERTIFICATE-----.+-----END CERTIFICATE-----)(.+)", java.util.regex.Pattern.DOTALL);
	    java.util.regex.Matcher matcher = pattern.matcher(rawCert);
	    if (matcher.matches()) {
  		authorCertStr = matcher.group(2);
	    }
	    
	    if (authorCertStr == null) {
		return null;
	    }
	    
	    cf = CertificateFactory.getInstance((String)(authorAttri.theAttributes.get("AuthorCertificateType")));
	    ByteArrayInputStream bais = new ByteArrayInputStream(authorCertStr.getBytes());
	    
  	    while (bais.available() > 0) {
  		authorCert = cf.generateCertificate(bais);
  	    }
 	}
	catch (Exception e) {
//  	    System.err.println(e); 
	    return null;
	}

	return authorCert;
    }


    /*
     * Retrieve the AuthorPublicKey from attribute file
     */
    public java.security.PublicKey getAuthorPublicKey(String id) {
	Certificate authorCert = this.getAuthorCertificate(id);
	if (authorCert == null) {
	    return null;
	}
	else {
	    java.security.PublicKey pKey = authorCert.getPublicKey();
	    return(pKey);
	}
    }


    /*
     * Retrieve the code for the class `name' from disk.  If
     * it does not exist there, it is loaded from the URL that was
     * specified. Get the class from the ZIP file.
     */
    public byte[] getCode(String id, String name) throws Exception {

	boolean methodExistsLocal = true;

	// Get the name of the document method we are in.
	String methodName;
	if (Thread.currentThread().getClass() ==
	    Class.forName("IndigoConnectionHandler")) {
	    // We are in a Connection-Handler, that means `name' is
	    // the name of the document method.
	    methodName = name;
	}
	else {
	    // We are not in a Connection-handler, therefore the
	    // name of the document method is not `name'. Instead it
	    // is stored in the currently active MethodThread().
	    MethodThread mt = (MethodThread)Thread.currentThread();
	    methodName = mt.methodName;
	}

	File methodCode = new File(getCodeName(id, methodName));
	if (methodCode.exists() == false) {
	    // Get the method location
	    methodExistsLocal = false;
	    Methods theMethods = getMethods(id);
	    URL location = (URL)theMethods.theMethods.get(methodName);
	    IndigoOut.errPrintln("   >>>> LOADING DM FROM REMOTE DM-SERVER: " + 
			       theMethods.theMethods.get(methodName)); 
	    if (location == null) {
		throw new IOException
		    ("Method " + name + " not found in document " + id);
	    }
	    // Get content of url and refresh the File object.
	    retrieveContent(location, getCodeName(id, methodName));
	    methodCode = new File(getCodeName(id, methodName));
	}
	
	IndigoOut.errPrintln("   >>>> Get " + name + 
			     " from: " + methodCode); 

  	JarFile zipFile;
	try {
  	    zipFile = new JarFile(methodCode, IndigoConfig.needMethodVerify());
	}
  	catch (SecurityException e) {
    	    throw new SecurityException(e.toString());
  	}
	catch (Exception e) {
    	    throw new IOException(e.toString());
	}


  	JarEntry classEntry;
	Enumeration enumJar = zipFile.entries();
	Certificate[] dmCertChain;
	Certificate authorCert;

	if (IndigoConfig.needMethodVerify()) {
	    try {
		// Get manifest and write its contents
  		Manifest manif = zipFile.getManifest();
		if (manif != null) {
  		    // java.util.jar.Attributes attr = manif.getMainAttributes();

		    // print the content-list of the zip-file
    		    // for (enumJar = zipFile.entries(); enumJar.hasMoreElements();) {
		    //    System.out.println(">> >> >> " + enumJar.nextElement());
  		    // }
		} 
		else {
		    throw new SecurityException("This method hasn't any manifest");
		}
	    }
	    catch (Exception e) {
		throw new SecurityException(e.toString());
	    }
	    
  	    classEntry = zipFile.getJarEntry(name + ".class");
	    
	    // JarEntry must be completely verified by reading from the entry 
	    // input stream until the end of the stream has been reached.
	    InputStream is = zipFile.getInputStream(classEntry);
	    int c;
	    while ((c = is.read()) != -1) {
		//do nothing here, just to make sure the whole stream is read
	    }
	    // This certificate array comprises all the signer certificates 
	    // that were used to verify this entry
	    dmCertChain = classEntry.getCertificates();
	    if (dmCertChain == null){
		IndigoOut.errPrintln("This method hasn't any certificate.");
    		throw new SecurityException("This method hasn't any certificate.");
	    }
	    
	    // Try to get the AuthorCert and AuthorPublicKey
	    // from the current document
	    java.security.PublicKey authorpKey = this.getAuthorPublicKey(id);
	    if (authorpKey == null) {
		IndigoOut.errPrintln("This document hasn't any correct AuthorCertificate for verifying of this method!");
		throw new SecurityException("This document hasn't any correct AuthorCertificate for verifying of this method!");
	    }

	    // Compare the public key from the authorcert with 
	    // the public key of the ducumentmethods signers. 
	    java.security.PublicKey dmpKey;
	    boolean keyEqual = false;
	    for (int i = 0; i < dmCertChain.length; i++) {
		dmpKey = dmCertChain[i].getPublicKey();
		// Determining If Two Keys Are Equal	
		if   ((authorpKey.equals(dmpKey)) || 
		      (Arrays.equals(authorpKey.getEncoded(), dmpKey.getEncoded()))) {
		    keyEqual = true;
		    break;
		}
	    }
	    if (keyEqual == false) {
		IndigoOut.errPrintln("This documentmethode was not signed from this author!");
		throw new SecurityException("This documentmethode was not signed from this author!");
	    }
	    IndigoOut.errPrintln("   >>>> This document method was correctly verified.");
    	}
 	else {
    	    classEntry = zipFile.getJarEntry(name + ".class");
  	}
	
	if (classEntry == null) {
	    throw new IOException(name + " not found in " + id);
	}

	// Read code into memory
	int   codeSize  = (int)classEntry.getSize();
	byte[] theCode  = new byte[(int)codeSize];
	
	int read = 0;
  	InputStream zipInput = zipFile.getInputStream(classEntry);
	while (read < codeSize) {
	    int newread = zipInput.read(theCode, read, codeSize - read);
	    if (newread == -1) {
		break;
	    }
	    read = read + newread;
	}
	zipInput.close();
	
	if (read < codeSize) {
	    throw new IOException("short read");
	}
	return theCode;
    }


    /*
     * Store the attributes of a meta document on disk
     */
    public void storeAttributes(String id, Attributes theAttributes)
	    throws IOException {

	PrintWriter attributeWriter;
	attributeWriter = new PrintWriter(
                new FileOutputStream(getAttributesName(id)));

	Enumeration attributeNames;
	attributeNames = theAttributes.theAttributes.keys();
	while (attributeNames.hasMoreElements()) {
	    String name;

	    try {
		name = (String)attributeNames.nextElement();
	    }
	    catch (NoSuchElementException e) {
		e.printStackTrace(System.err);
		return;
	    }
	    attributeWriter.println(name + " "
	            + (String)theAttributes.theAttributes.get(name));
	}
	attributeWriter.close();
    }


    /*
     * Store the method information of a meta document on disk
     */
    public void storeMethods(String id, Methods theMethods)
	    throws IOException {

	PrintWriter methodWriter;
        methodWriter = new PrintWriter(
                new FileOutputStream(getMethodsName(id)));

	Enumeration methodNames;
	methodNames = theMethods.theMethods.keys();
	while (methodNames.hasMoreElements()) {
	    String name;

	    try {
		name = (String)methodNames.nextElement();
	    }
	    catch (NoSuchElementException e) {
		e.printStackTrace(System.err);
		return;
	    }
	    methodWriter.println(name + " "
	            + ((URL)theMethods.theMethods.get(name)).toString()
		    + " application/java");
	}
	methodWriter.close();
    }


    /*
     * Read a meta document from an input stream and store it on disk
     * the meta document store.
     */
    public boolean decode(String id, BoundaryReader input) {

	try {

	    /*
	     * Create two directories, the root of the document-files,
	     * e.g. attributes, methods, and content and the root of
	     * its method-files which store the java classes.
	     */
	    new File(getDocumentDir(id)).mkdir();
	    new File(getCodeDir(id)).mkdir();

	    // Read header and check content type
	    Header aHeader;
	    aHeader = new Header(input);
	    String contentType = (String)aHeader.getValue("content-type");
	    if (contentType == null ||
		contentType.equals("application/x-metadoc") == false) {
		errorMessage = "Bad content type" + contentType;
		return false;
	    }

	    // Get the boundary
	    String boundary = aHeader.getBoundary();
	    if (boundary == null) {
		errorMessage  = "No boundary specified";
		return false;
	    }
	    
	    // Push the boundary and read until we hit it
	    input.pushBoundary(boundary);
	    String line = input.readLine();
	    while (line != null) {
		line = input.readLine();
	    }

	    // Skip boundary into attribute section and read attributes
	    if (input.isBoundary() == false) {
		errorMessage  = "Bad message format";
		return false;
	    }
	    input.skipBoundary();
	    // Read header and check content type
	    aHeader = new Header(input);
	    contentType = (String)aHeader.getValue("content-type");
	    if (contentType == null ||
		contentType.equals("application/x-metadoc-attributes")
		== false) {
		errorMessage = "Bad content type" + contentType;
		return false;
	    }
	    Attributes theAttributes = new Attributes();
	    theAttributes.readFrom(input);
	    storeAttributes(id, theAttributes);

	    // Skip boundary into method section and read attributes
	    if (input.isBoundary() == false) {
		errorMessage  = "Bad message format";
		return false;
	    }
	    input.skipBoundary();
	    // Read header and check content type
	    aHeader = new Header(input);
	    contentType = (String)aHeader.getValue("content-type");
	    if (contentType == null ||
		contentType.equals("application/x-metadoc-methods")
		== false) {
		errorMessage = "Bad content type" + contentType;
		return false;
	    }
	    Methods theMethods = new Methods();
	    theMethods.readFrom(input);
	    storeMethods(id, theMethods);

	    // Skip boundary into content section and read content
	    if (input.isBoundary() == false) {
		errorMessage  = "Bad message format";
		return false;
	    }
	    input.skipBoundary();
	    // Read header and check content type
	    aHeader = new Header(input);
	    contentType = (String)aHeader.getValue("content-type");
	    if (contentType == null ||
		contentType.equals("application/x-metadoc-content")
		== false) {
		errorMessage = "Bad content type " + contentType;
		return false;
	    }
	    PrintWriter aWriter = new PrintWriter(
                    new FileOutputStream(getContentName(id)));
	    line = input.readLine();
	    while (line != null) {
		aWriter.println(line);
		line = input.readLine();
	    }
	    aWriter.close();

	    // Check for final boundary
	    if (input.isFinalBoundary() == false) {
		errorMessage  = "To many parts in meta document";
		return false;
	    }
	    errorMessage = "No error";
	    return true;
	}
	catch (Exception e) {
	    e.printStackTrace(System.err);
	    errorMessage = e.toString() + ": " + e.getMessage();
	    return false;
	}
    }
}
