/*
 *  MethodLoader.java  $Revision: 2.7 $
 *
 *  $Date: 1999/12/21 14:53:34 $ $Author: cristian $
 *
 *  Copyright (C) 1999  Christian Moench
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Christian Moench
 *  Dept. of Computer Science
 *  Johann Wolfgang Goethe-University
 *  60325 Frankfurt, Germany
 *  Tel.  +49 69 79828358
 *  Fax.  +49 69 79823340
 *
 *  $Log: MethodLoader.java,v $
 *  Revision 2.7  1999/12/21 14:53:34  cristian
 *  added multiplexed standard output for document methods
 *
 *  Revision 2.6  1999/12/19 16:14:55  razi
 *  Including IndigoConfig.java
 *
 *  Revision 2.2  1999/12/15 16:37:23  cristian
 *  cleanups
 *
 *  Revision 1.3  1999/12/13 21:13:11  cristian
 *  added support for zip-files containing multiple classes
 *
 *  Revision 1.2  1999/12/13 18:30:08  cristian
 *  new method scheme started
 *
 *  Revision 1.1.1.1  1999/07/28 15:55:58  cristian
 *  Initial import
 *
 *
 */
import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;


public class MethodLoader extends ClassLoader {

    // The id of the document is stored here
    protected String            documentId;
    protected MetaDocumentStore theStore;
    protected Hashtable         theCache;

    /*
     * Overwrite loadClass-method to ensure access to our loader.
     */
    public Class loadClass(String name) 
	throws SecurityException, ClassNotFoundException {
	return loadClass(name, true);
    }

    /*
     * Load the class that implements the method `name'. If it is not
     * found in the current document, assume it is a system class and
     * delegate the load-Operation to the JVM.
     */
    public synchronized Class loadClass(String name, boolean resolve)
	    throws SecurityException, ClassNotFoundException {

	Class  aClass;
	byte[] classStorage;

//  	IndigoOut.errPrintln("   >>>> Get from loadClass:" + name);

	// Try to find a system class
	try {
	    aClass = findSystemClass(name);
	    return aClass;
	} catch (Exception e) {}
	
	// Try to get a class from the cache.
	aClass = (Class)theCache.get(name);

	// Try to load a class from a document store.
	if (aClass == null) {
	    try {
		classStorage = theStore.getCode(documentId, name);
	    } 
	    catch (SecurityException e) {
		throw new SecurityException(e.getMessage());
	    }
	    catch (Exception e) {
		throw new ClassNotFoundException(e.getMessage());
	    }
	    // Build a class from the data we read.
	    aClass = defineClass(name, classStorage, 0, classStorage.length);
	    if (resolve == true) {
		resolveClass(aClass);
	    }
	    // Put the class into the cache
	    theCache.put(name, aClass);
	}
	return aClass;
    }

    public MethodLoader(MetaDocumentStore aStore, String id) {
	super();
	documentId = id;
	theStore   = aStore;
	theCache   = new Hashtable();
    }
}
