/*
 *  MethodThread.java  $Revision: 2.7 $
 *
 *  $Id: MethodThread.java,v 2.7 1999/12/21 14:53:34 cristian Exp $
 *  $Date: 1999/12/21 14:53:34 $
 *
 *  Copyright (C) 1999  Christian Moench
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Christian Moench
 *  Dept. of Computer Science
 *  Johann Wolfgang Goethe-University
 *  60325 Frankfurt, Germany
 *  Tel.  +49 69 79828358
 *  Fax.  +49 69 79823340
 *
 *  $Log: MethodThread.java,v $
 *  Revision 2.7  1999/12/21 14:53:34  cristian
 *  added multiplexed standard output for document methods
 *
 *  Revision 2.6  1999/12/19 16:14:55  razi
 *  Including IndigoConfig.java
 *
 *  Revision 2.3  1999/12/19 16:07:23  razi
 *  IndigoConfig.java was added.
 *
 *  Revision 2.2  1999/12/15 16:37:23  cristian
 *  cleanups
 *
 *  Revision 1.4  1999/12/15 10:21:46  cristian
 *  Security now based on thread groups, all untrusted groups are restricted,
 *  this supports for multiple threads in document methods
 *
 *  Revision 1.3  1999/12/13 21:13:11  cristian
 *  added support for zip-files containing multiple classes
 *
 *  Revision 1.2  1999/12/13 18:30:08  cristian
 *  new method scheme started
 *
 *  Revision 1.1  1999/08/05 12:35:23  cristian
 *  initial version
 *
 *
 */
import java.lang.*;
import java.lang.reflect.*;
import java.net.*;
import javax.net.ssl.*;
import javax.security.cert.X509Certificate;

public class MethodThread extends Thread {

    // Stores the active document store and the id of the document.
    public MetaDocumentStore documentStore;
    public String            documentId;
    public String            methodName;
    public Socket            clientSocket;

    protected Method      theMethod;
    protected Object[]    theArguments;
    protected Throwable   theException;

    public MethodThread(ThreadGroup aGroup,
			String docid, String aName, Method aMethod,
			Object[] someArguments,
			MetaDocumentStore docStore) {
	super(aGroup, null, docid + "#" + aName + "-main thread");
	theMethod     = aMethod;
	theArguments  = someArguments;
	methodName    = aName;
	documentId    = docid;
	documentStore = docStore;
    }


    public void run() {
	theException = null;
	try {
	    theMethod.invoke(null, theArguments);
	}
	catch (InvocationTargetException anException) {
	    theException = anException.getTargetException();
	    return;
	}
	catch (IllegalAccessException anException) {
	    theException = anException;
	}
    }


    public Throwable getException() {
	return theException;
    }
}
