/*
 *  Methods.java  $Revision: 2.6 $
 *
 *  $Date: 1999/12/19 16:14:55 $ $Author: razi $
 *
 *  Copyright (C) 1999  Christian Moench
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Christian Moench
 *  Dept. of Computer Science
 *  Johann Wolfgang Goethe-University
 *  60325 Frankfurt, Germany
 *  Tel.  +49 69 79828358
 *  Fax.  +49 69 79823340
 *
 *  $Log: Methods.java,v $
 *  Revision 2.6  1999/12/19 16:14:55  razi
 *  Including IndigoConfig.java
 *
 *  Revision 2.2  1999/12/15 16:37:24  cristian
 *  cleanups
 *
 *  Revision 1.1.1.1  1999/07/28 15:55:58  cristian
 *  Initial import
 *
 *
 */
import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;



public class Methods {


    Hashtable theMethods;


    /*
     * Read methods specification from a buffered reader.
     */
    public void readFrom(BufferedReader reader) throws IOException {
	URL             anUrl;
	String          line, name, location, type;
	StringTokenizer splitter;


	line = reader.readLine();
	while (line != null) {

	    splitter = new StringTokenizer(line.trim());
	    if (splitter.countTokens() != 3) {
		throw new IOException("Bad method line: " + line);
	    }

	    // Read method name, location, and type.
	    try {
		name     = splitter.nextToken();
		location = splitter.nextToken();
		type     = splitter.nextToken();
	    }
	    catch (NoSuchElementException e) {
		e.printStackTrace(System.err);
		return;
	    }

	    if (type.equals("application/java") == false) {
		throw new IOException("Unsupported mime type: " + type);
	    }

	    try {
		anUrl = new URL(location);
	    }
	    catch (MalformedURLException e) {
		e.printStackTrace(System.err);
		throw new IOException("malformed URL: " + location);
	    }
	    theMethods.put(name, anUrl);
	    line = reader.readLine();
	}
    }


    /*
     * Write the method specification to a print writer.
     */
    public void writeTo(PrintWriter writer) throws IOException {
	URL         location;
	String      name;
	Enumeration methodNames;


	methodNames = theMethods.keys();
	while (methodNames.hasMoreElements()) {
	    try {
		name = (String)methodNames.nextElement();
	    }
	    catch (NoSuchElementException e) {
		e.printStackTrace(System.err);
		return;
	    }
	    location = (URL)theMethods.get(name);
	    writer.println(name + " " + location.toString()
			   + " application/java");
	}
    }


    public Methods() {
	theMethods = new Hashtable();
	return;
    }
}
